@@ -1,5 +1,5 @@
 import os, os.path
-import htmllib, HTMLParser
+import htmllib, HTMLParser, formatter
 import time
 import logging
 import traceback
@@ -15,12 +15,13 @@ GTKHTML=0
 MOZILLA=1
 
 class EntryFormatter:
-	def __init__(self, mm=None, with_feed_titles=False, indicate_new=False, basic_progress=False, ajax_url=None):
+	def __init__(self, mm=None, with_feed_titles=False, indicate_new=False, basic_progress=False, ajax_url=None, renderer=MOZILLA):
 		self._mm = mm
 		self._with_feed_titles = with_feed_titles
 		self._indicate_new = indicate_new
 		self._basic_progress = basic_progress
 		self._ajax_url = ajax_url
+		self._renderer = renderer
 		
 	def htmlify_item(self, item, convert_newlines=False):
 		""" Take an item as returned from ptvDB and turn it into an HTML page.  Very messy at times,
@@ -31,11 +32,20 @@ class EntryFormatter:
 		#ret.append('<div class="heading">')
 		if self._indicate_new:
 			if item['new']:
-				ret.append("""<div class="entry_new" oncontextmenu="javascript:parent.location='rightclick:%i'">""" % item['entry_id'])
+				javascript = ""
+				if self._renderer == MOZILLA:
+					javascript = """oncontextmenu="javascript:parent.location='rightclick:%i'" """ % item['entry_id']
+				ret.append("""<div class="entrynew" %s>""" % javascript)
 			else:
-				ret.append("""<div class="entry_old" oncontextmenu="javascript:parent.location='rightclick:%i'">""" % item['entry_id'])
+				javascript = ""
+				if self._renderer == MOZILLA:
+					javascript = """oncontextmenu="javascript:parent.location='rightclick:%i'" """ % item['entry_id']
+				ret.append("""<div class="entryold" %s>""" % javascript)
 		else:
-			ret.append("""<div class="entry" oncontextmenu="javascript:parent.location='rightclick:%i'">""" % item['entry_id'])
+			javascript = ""
+			if self._renderer == MOZILLA:
+				javascript = """oncontextmenu="javascript:parent.location='rightclick:%i'" """ % item['entry_id']
+			ret.append("""<div class="entrynew" %s>""" % javascript)
 
 		ret.append('''<table style="text-align: left; width: 100%;" border="0" cellpadding="0" cellspacing="0"><tr><td>''')
 
@@ -56,11 +66,20 @@ class EntryFormatter:
 		ret.append('</td><td style="text-align: right;">')
 
 		if not utils.RUNNING_SUGAR:
-			cb_status = item['keep'] and "CHECKED" or "UNCHECKED"
-			cb_function = item['keep'] and "unkeep" or "keep"
-	
-			ret.append('''<form id="keep"> <input type="checkbox" id="keep" name="keep" class="radio" onclick="parent.location='%s:%i'" %s="yes"><a href="%s:%i">%s</a></form>''' % 
+			if self._renderer == MOZILLA:
+				cb_status = item['keep'] and "CHECKED" or "UNCHECKED"
+				cb_function = item['keep'] and "unkeep" or "keep"
+		
+				ret.append('''<form id="keep"> <input type="checkbox" id="keep" name="keep" class="radio" onclick="parent.location='%s:%i'" %s="yes"><a href="%s:%i">%s</a></form>''' % 
 			           (cb_function, item['entry_id'], cb_status, cb_function, item['entry_id'], _('Keep New')))
+			elif self._renderer == GTKHTML:
+				cb_function = item['keep'] and "unkeep" or "keep"
+				if not item['keep']:
+					link_name = _('Keep New')
+				else:
+					link_name = _("Don't Keep New")
+				ret.append('''<a href="%s:%i">%s</a>''' % 
+			           (cb_function, item['entry_id'], link_name))
 
 		ret.append('</td></tr></table>')
 
@@ -72,6 +91,7 @@ class EntryFormatter:
 			#ret.append('</div>')
 
 		if item.has_key('media'):
+			item['media'].sort()
 			for medium in item['media']:
 				ret += self.htmlify_media(medium)
 		ret.append('<div class="content">')
@@ -82,10 +102,10 @@ class EntryFormatter:
 				ret.append('%s' % item['description'])
 		ret.append('</div>')
 		if item.has_key('link'):
-			ret.append('<a href="' + item['link'] + '">' + _("Full Entry...") + '</a>')
+			ret.append('<a href="' + item['link'] + '">' + _("Open in Browser...") + '</a>')
 		ret.append('</p></div>')
 		#print "\n".join(ret)
-		return "".join(ret)
+		return "\n".join(ret)
 	
 	def htmlify_media(self, medium):
 		ret = []
@@ -98,13 +118,13 @@ class EntryFormatter:
 		elif medium['download_status'] == D_DOWNLOADING: 
 			if self._basic_progress:
 				if self._ajax_url is None:
-					ret.append('<img src="file://' + os.path.join(utils.get_glade_prefix(), "pixmaps", "throbber.gif") + '"/>')
+					ret.append('<img src="file://' + utils.get_image_path("throbber.gif") + '"/>')
 				else:
 					ret.append('<img src="' + self._ajax_url + '/pixmaps/throbber.gif"/>')
 				ret.append('<i>'+_('Downloading %s...') % utils.format_size(medium['size'])+'</i> '+self._html_command('pause:',medium['media_id'])+' '+self._html_command('stop:',medium['media_id']))
 			elif medium.has_key('progress_message'): #downloading and we have a custom message
 				if self._ajax_url is None:
-					ret.append('<img src="file://' + os.path.join(utils.get_glade_prefix(), "pixmaps", "throbber.gif") + '"/>')
+					ret.append('<img src="file://' + utils.get_image_path("throbber.gif") + '"/>')
 				else:
 					ret.append('<img src="' + self._ajax_url + '/pixmaps/throbber.gif"/>')
 				ret.append('<p><i>'+medium['progress_message']+'</i></p>')
@@ -120,7 +140,7 @@ class EntryFormatter:
 					ret.append('''<table border="0" cellpadding="0" cellspacing="12pt">''')
 					ret.append('''<tr><td rowspan="2">''')
 					if self._ajax_url is None:
-						ret.append('<img src="file://' + os.path.join(utils.get_glade_prefix(), "pixmaps", "throbber.gif") + '"/>')
+						ret.append('<img src="file://' + utils.get_image_path("throbber.gif") + '"/>')
 					else:
 						ret.append('<img src="' + self._ajax_url + '/pixmaps/throbber.gif"/>')
 					ret.append("</td><td>")
@@ -139,7 +159,7 @@ class EntryFormatter:
 				ret.append('''<table border="0" cellpadding="0" cellspacing="12pt">''')
 				ret.append('''<tr><td rowspan="2">''')
 				if self._ajax_url is None:
-					ret.append('<img src="file://' + os.path.join(utils.get_glade_prefix(), "pixmaps", "throbber.gif") + '"/>')
+					ret.append('<img src="file://' + utils.get_image_path("throbber.gif") + '"/>')
 				else:
 					ret.append('<img src="' + self._ajax_url + '/pixmaps/throbber.gif"/>')
 				ret.append("</td><td>")
@@ -257,23 +277,23 @@ class EntryFormatter:
 		ret.append("""</td></tr></table>""")
 		return "\n".join(ret)
 
-class HTMLimgParser(htmllib.HTMLParser):
-	def __init__(self):
-		htmllib.HTMLParser.__init__(self, formatter.NullFormatter())
-		self.images=[]
-		
-	def do_img(self, attributes):
-		for name, value in attributes:
-			if name == 'src':
-				new_image = value
-				self.images.append(new_image)
+#class HTMLimgParser(htmllib.HTMLParser):
+#	def __init__(self):
+#		htmllib.HTMLParser.__init__(self, formatter.NullFormatter())
+#		self.images=[]
+#		
+#	def do_img(self, attributes):
+#		for name, value in attributes:
+#			if name == 'src':
+#				new_image = value
+#				self.images.append(new_image)
 				
 class HTMLHighlightParser(HTMLParser.HTMLParser):
 	def __init__(self, highlight_terms):
 		HTMLParser.HTMLParser.__init__(self)
 		highlight_terms = highlight_terms.replace('"','')
 		highlight_terms = highlight_terms.replace("'",'')
-		self.terms = [a.upper() for a in highlight_terms.split() if len(a)>2]
+		self.terms = [a.upper() for a in highlight_terms.split() if len(a)>2 and a.upper() not in ['AND','NOT']]
 		self.new_data = ""
 		self.style_start="""<span style="background-color: #ffff00">"""
 		self.style_end  ="</span>"
