@@ -15,7 +15,8 @@ import utils
 import IconManager
 import MainWindow
 
-
+if utils.RUNNING_HILDON:
+	import hildon
 
 NONE=-1 #unused, needs a value
 ALL=0
@@ -58,6 +59,9 @@ class FeedList(gobject.GObject):
 		'feed-selected': (gobject.SIGNAL_RUN_FIRST, 
                            gobject.TYPE_NONE, 
                            ([gobject.TYPE_INT])),
+        'feed-clicked': (gobject.SIGNAL_RUN_FIRST, 
+                           gobject.TYPE_NONE, 
+                           ([])),
 		'search-feed-selected': (gobject.SIGNAL_RUN_FIRST, 
                            gobject.TYPE_NONE, 
                            ([gobject.TYPE_INT])),                  
@@ -69,12 +73,13 @@ class FeedList(gobject.GObject):
                            ([gobject.TYPE_INT]))
 	}
                            
-	def __init__(self, widget_tree, app, db, fancy=False):
+	def __init__(self, widget_tree, app, fancy=False):
 		gobject.GObject.__init__(self)
 		self._app = app
-		self._db = db
-		self._icon_manager = IconManager.IconManager(self._db.home)
+		self._icon_manager = IconManager.IconManager(self._app.db.home)
 		self._scrolled_window = widget_tree.get_widget('feed_scrolled_window')
+		if utils.RUNNING_HILDON:
+			hildon.hildon_helper_set_thumb_scrollbar(self._scrolled_window, True)
 		self._va = self._scrolled_window.get_vadjustment()
 		self._widget = widget_tree.get_widget('feedlistview')
 		self._entry_list_widget = widget_tree.get_widget('entrylistview')
@@ -104,6 +109,7 @@ class FeedList(gobject.GObject):
 		self._fancy = fancy
 		self.__widget_width = 0
 		self.__resetting_columns = False
+		self.__displayed_context_menu = False #for hildon
 		
 		#build list view
 		
@@ -162,6 +168,7 @@ class FeedList(gobject.GObject):
 		self._widget.get_selection().connect("changed", self._item_selection_changed)
 		self._widget.connect("row-activated", self.on_row_activated)
 		self._widget.connect("button-press-event", self._on_button_press_event)
+		self._widget.connect("button-release-event", self._on_button_release_event)
 		if utils.RUNNING_HILDON:
 			self._widget.tap_and_hold_setup(menu=self._get_context_menu(False))
 		
@@ -185,8 +192,10 @@ class FeedList(gobject.GObject):
 		
 		#init style
 		if self._fancy:
-			if utils.RUNNING_SUGAR or utils.RUNNING_HILDON:
+			if utils.RUNNING_SUGAR:
 				self._icon_renderer.set_property('stock-size',gtk.ICON_SIZE_SMALL_TOOLBAR)
+			elif utils.RUNNING_HILDON:
+				self._icon_renderer.set_property('stock-size',gtk.ICON_SIZE_BUTTON)
 			else:
 				self._icon_renderer.set_property('stock-size',gtk.ICON_SIZE_LARGE_TOOLBAR)
 			self._widget.set_property('rules-hint', True)
@@ -217,7 +226,9 @@ class FeedList(gobject.GObject):
 		self.filter_all(False)
 	
 	def __entry_updated_cb(self, app, entry_id, feed_id):
-		self.update_feed_list(feed_id,['readinfo','icon'])	
+		self.update_feed_list(feed_id,['readinfo','icon'])
+		for f in self._app.db.get_pointer_feeds(feed_id):
+			self.update_feed_list(f,['readinfo','icon'])
 		
 	#def __entry_selected_cb(self, feed_id, entry_id):
 	#	self.mark_entries_read(feed_id, 1)
@@ -227,15 +238,27 @@ class FeedList(gobject.GObject):
 		#logging.debug("feedlist entries viewed")
 		for feed_id, id_list in viewlist:
 			self.mark_entries_read(len(id_list), feed_id)
+			
+		for f in self._app.db.get_associated_feeds(feed_id):
+			if f != feed_id:
+				self.update_feed_list(f, ['readinfo','icon'])
 		
 	def __entries_unviewed_cb(self, app, viewlist):
 		for feed_id, id_list in viewlist:
 			self.mark_entries_read(0 - len(id_list), feed_id)
-		
+			
+		for f in self._app.db.get_associated_feeds(feed_id):
+			if f != feed_id:
+				self.update_feed_list(f, ['readinfo','icon'])
+						
 	def __update_feed_count_cb(self, o, feed_id, count):
 		#logging.debug("update %i ->  %i" % (feed_id, count))
 		self.update_feed_list(feed_id, update_what=['readinfo'], 
 			update_data={'unread_count':count})
+			
+		for f in self._app.db.get_associated_feeds(feed_id):
+			if f != feed_id:
+				self.update_feed_list(f, ['readinfo','icon'])
 
 	def grab_focus(self):
 		self._widget.grab_focus()
@@ -247,12 +270,12 @@ class FeedList(gobject.GObject):
 		#	self._articles_column.set_visible(False)
 		if len(self._feedlist)==0:
 			#first fill out rough feedlist
-			db_feedlist = self._db.get_feedlist()
+			db_feedlist = self._app.db.get_feedlist()
 			blank_pixbuf = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB,True,8, 10,10)
 			blank_pixbuf.fill(0xffffff00)			
 			for feed_id,title,url in db_feedlist:
 				if utils.RUNNING_HILDON:
-					title_m = '<span size="x-small">%s</span>' % title 	 
+					title_m = '<span size="large">%s</span>' % title 	 
 				elif self._fancy:
 					title_m = title+"\n"
 				else:
@@ -279,8 +302,8 @@ class FeedList(gobject.GObject):
 		"""A generator that updates the feed list.  Called from populate_feeds"""	
 		selection = self._widget.get_selection()
 		#selected = self.get_selected()
-		feed_cache = self._db.get_feed_cache()
-		db_feedlist = self._db.get_feedlist()
+		feed_cache = self._app.db.get_feed_cache()
+		db_feedlist = self._app.db.get_feedlist()
 		
 		blank_pixbuf = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB,True,8, 10,10)
 		blank_pixbuf.fill(0xffffff00)
@@ -334,14 +357,14 @@ class FeedList(gobject.GObject):
 					#bad cache, trigger test below
 					entry_count = None
 			else:
-				feed_info   = self._db.get_feed_verbose(feed_id)
+				feed_info   = self._app.db.get_feed_verbose(feed_id)
 				unviewed    = feed_info['unread_count']
 				flag        = feed_info['important_flag']
 				pollfail    = feed_info['poll_fail']
 				entry_count = feed_info['entry_count']
 				m_first_entry_title = ""
 			if entry_count==0 or entry_count is None: #this is a good indication that the cache is bad
-				feed_info   = self._db.get_feed_verbose(feed_id)
+				feed_info   = self._app.db.get_feed_verbose(feed_id)
 				unviewed    = feed_info['unread_count']
 				flag        = feed_info['important_flag']
 				pollfail    = feed_info['poll_fail']
@@ -363,14 +386,18 @@ class FeedList(gobject.GObject):
 			if self._fancy:
 				if visible:
 					if len(m_first_entry_title) == 0:
-						m_first_entry_title = self._db.get_first_entry_title(feed_id, True)
+						m_first_entry_title = self._app.db.get_first_entry_title(feed_id, True)
 					m_details_loaded = True
 				else:
 					if len(m_first_entry_title) > 0:
 						m_details_loaded = True
 					else:
 						m_details_loaded = False
-				m_pixbuf = self._icon_manager.get_icon_pixbuf(feed_id) #, 
+				if utils.RUNNING_HILDON:
+					m_pixbuf = self._icon_manager.get_icon_pixbuf(feed_id , 
+								    64, 64, MIN_SIZE, MIN_SIZE)	
+				else:
+					m_pixbuf = self._icon_manager.get_icon_pixbuf(feed_id) #, 
 								    #MAX_WIDTH, MAX_HEIGHT, MIN_SIZE, MIN_SIZE)
 				model, iter = selection.get_selected()
 				try: sel = model[iter][FEEDID]
@@ -500,7 +527,7 @@ class FeedList(gobject.GObject):
 		
 		if 'title' in update_what or 'icon' in update_what:
 			if not update_data.has_key('flag_list'):
-				update_data['flag_list'] = self._db.get_entry_flags(feed_id)
+				update_data['flag_list'] = self._app.db.get_entry_flags(feed_id)
 						
 			updated=0
 			unviewed=0
@@ -514,13 +541,17 @@ class FeedList(gobject.GObject):
 			flag = self._pick_important_flag(feed_id, update_data['flag_list'])				
 
 		if 'image' in update_what and self._fancy:
-			feed[PIXBUF] = self._icon_manager.get_icon_pixbuf(feed_id) #, 
+			if utils.RUNNING_HILDON:
+				feed[PIXBUF] = self._icon_manager.get_icon_pixbuf(feed_id, 
+							64, 64, MIN_SIZE, MIN_SIZE)
+			else:
+				feed[PIXBUF] = self._icon_manager.get_icon_pixbuf(feed_id) #, 
 							#MAX_WIDTH, MAX_HEIGHT, MIN_SIZE, MIN_SIZE)
 			
 		if 'readinfo' in update_what:
-			#db_unread_count = self._db.get_unread_count(feed_id) #need it always for FIXME below
+			#db_unread_count = self._app.db.get_unread_count(feed_id) #need it always for FIXME below
 			if not update_data.has_key('unread_count'):
-				update_data['unread_count'] = self._db.get_unread_count(feed_id)#, db_unread_count)
+				update_data['unread_count'] = self._app.db.get_unread_count(feed_id)#, db_unread_count)
 			if update_data['unread_count'] > 0:
 				if feed[FLAG] & ptvDB.F_UNVIEWED==0:
 					feed[FLAG] = feed[FLAG] + ptvDB.F_UNVIEWED
@@ -530,7 +561,7 @@ class FeedList(gobject.GObject):
 			feed[UNREAD]   = update_data['unread_count']
 			feed[TOTAL]    = len(update_data['flag_list'])
 
-			if utils.RUNNING_HILDON:
+			if utils.RUNNING_HILDON and not self._fancy:
 				readinfo_string = "(%d)" % (update_data['unread_count'],)
 			else:
 				readinfo_string = "(%d/%d)" % (update_data['unread_count'], len(update_data['flag_list']))
@@ -548,13 +579,13 @@ class FeedList(gobject.GObject):
 		if 'title' in update_what:
 			selected = self.get_selected()
 			if not update_data.has_key('title'):
-				update_data['title'] = self._db.get_feed_title(feed_id)
+				update_data['title'] = self._app.db.get_feed_title(feed_id)
 			old_title_len = len(feed[TITLE])
 			new_title_len = len(update_data['title'])
 			
 			# don't update feed[TITLE] yet, we need these data first
 			if self._fancy:
-				try: feed[FIRSTENTRYTITLE] = self._db.get_first_entry_title(feed_id, True)
+				try: feed[FIRSTENTRYTITLE] = self._app.db.get_first_entry_title(feed_id, True)
 				except: feed[FIRSTENTRYTITLE] = ""
 				feed[MARKUPTITLE] = self._get_fancy_markedup_title(update_data['title'],feed[FIRSTENTRYTITLE],feed[UNREAD], feed[TOTAL], flag, feed[FEEDID])
 			else:
@@ -564,7 +595,7 @@ class FeedList(gobject.GObject):
 				feed[TITLE] = update_data['title']
 				try:
 					old_iter = self._feedlist.get_iter((self.find_index_of_item(feed_id),))
-					new_iter = self._feedlist.get_iter(([f[0] for f in self._db.get_feedlist()].index(feed_id),))
+					new_iter = self._feedlist.get_iter(([f[0] for f in self._app.db.get_feedlist()].index(feed_id),))
 					self._feedlist.move_after(old_iter,new_iter)
 					if selected == feed_id:
 						self._widget.scroll_to_cell((self.find_index_of_item(feed_id),))
@@ -578,7 +609,7 @@ class FeedList(gobject.GObject):
 			
 		if 'icon' in update_what:
 			if not update_data.has_key('pollfail'):
-				update_data['pollfail'] = self._db.get_feed_poll_fail(feed_id)
+				update_data['pollfail'] = self._app.db.get_feed_poll_fail(feed_id)
 			feed[POLLFAIL] = update_data['pollfail']
 			feed[STOCKID] = self._get_icon(flag)
 			if update_data['pollfail']:
@@ -634,7 +665,7 @@ class FeedList(gobject.GObject):
 		if feed[UNREAD] > 0 and feed[FLAG] & ptvDB.F_UNVIEWED == 0:
 			feed[FLAG] += ptvDB.F_UNVIEWED
 		
-		if utils.RUNNING_HILDON:
+		if utils.RUNNING_HILDON and not self._fancy:
 			readinfo_string = "(%d)" % (feed[UNREAD],)
 		else:
 			readinfo_string = "(%d/%d)" % (feed[UNREAD], feed[TOTAL])
@@ -760,8 +791,9 @@ class FeedList(gobject.GObject):
 		selected = self.get_selected()
 		index = self.find_index_of_item(selected)
 		
-		if self.filter_setting > SEARCH:
-			feeds_with_tag = self._db.get_feeds_for_tag(self.filter_name)
+		#stupid exception for when we don't have search
+		if self.filter_setting > SEARCH:# or (not utils.HAS_SEARCH and self.filter_setting == SEARCH):
+			feeds_with_tag = self._app.db.get_feeds_for_tag(self.filter_name)
 			
 		i=-1
 		for feed in self._feedlist:
@@ -773,13 +805,13 @@ class FeedList(gobject.GObject):
 				if flag & ptvDB.F_DOWNLOADED or flag & ptvDB.F_PAUSED:
 					passed_filter = True
 			elif self.filter_setting == NOTIFY:
-				opts = self._db.get_flags_for_feed(feed[FEEDID])
+				opts = self._app.db.get_flags_for_feed(feed[FEEDID])
 				if opts & ptvDB.FF_NOTIFYUPDATES:
 					passed_filter = True
 			elif self.filter_setting == ALL:
 				passed_filter = True
 			else:
-				#tags = self._db.get_tags_for_feed(feed[FEEDID])
+				#tags = self._app.db.get_tags_for_feed(feed[FEEDID])
 				#if tags:
 				#	if self.filter_name in tags:
 				#		passed_filter = True
@@ -831,13 +863,13 @@ class FeedList(gobject.GObject):
 			if flag & ptvDB.F_DOWNLOADED or flag & ptvDB.F_PAUSED:
 				passed_filter = True
 		elif self.filter_setting == NOTIFY:
-			opts = self._db.get_flags_for_feed(feed[FEEDID])
+			opts = self._app.db.get_flags_for_feed(feed[FEEDID])
 			if opts & ptvDB.FF_NOTIFYUPDATES:
 				passed_filter = True
 		elif self.filter_setting == ALL:
 			passed_filter = True
 		else:
-			tags = self._db.get_tags_for_feed(feed[FEEDID])
+			tags = self._app.db.get_tags_for_feed(feed[FEEDID])
 			if tags:
 				if self.filter_name in tags:
 					passed_filter = True
@@ -884,7 +916,7 @@ class FeedList(gobject.GObject):
 
 			if (row[VISIBLE] or not visible_only) and not row[DETAILS_LOADED]:
 				then = time.time()
-				try: row[FIRSTENTRYTITLE] = self._db.get_first_entry_title(row[FEEDID], True)
+				try: row[FIRSTENTRYTITLE] = self._app.db.get_first_entry_title(row[FEEDID], True)
 				except: row[FIRSTENTRYTITLE] = ""
 				now = time.time()
 				if now - then > 2 and not visible_only:
@@ -925,13 +957,13 @@ class FeedList(gobject.GObject):
 			if flag & ptvDB.F_DOWNLOADED or flag & ptvDB.F_PAUSED:
 				passed_filter = True
 		elif self.filter_setting == NOTIFY:
-			opts = self._db.get_flags_for_feed(feed[FEEDID])
+			opts = self._app.db.get_flags_for_feed(feed[FEEDID])
 			if opts & ptvDB.FF_NOTIFYUPDATES:
 				passed_filter = True
 		elif self.filter_setting == ALL:
 			passed_filter = True
 		else:
-			tags = self._db.get_tags_for_feed(feed_id)
+			tags = self._app.db.get_tags_for_feed(feed_id)
 			if tags:
 				if self.filter_name in tags:
 					passed_filter = True
@@ -943,7 +975,7 @@ class FeedList(gobject.GObject):
 			return
 		index = path[0]
 		model = treeview.get_model()
-		link = self._db.get_feed_info(model[index][FEEDID])['link']
+		link = self._app.db.get_feed_info(model[index][FEEDID])['link']
 		if link is None:
 			dialog = gtk.Dialog(title=_("No Homepage"), parent=None, flags=gtk.DIALOG_MODAL, buttons=(gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
 			label = gtk.Label(_("There is no homepage associated with this feed.  You can set one in the feed properties."))
@@ -1008,7 +1040,7 @@ class FeedList(gobject.GObject):
 		self._feedlist.clear()
 		
 	def add_feed(self, feed_id):
-		newlist = self._db.get_feedlist()
+		newlist = self._app.db.get_feedlist()
 		index = [f[0] for f in newlist].index(feed_id)
 		feed = newlist[index]
 		#print "-----------ADDFEED---------"
@@ -1033,6 +1065,13 @@ class FeedList(gobject.GObject):
 		if event.button==3: #right click
 			self.do_context_menu(widget, event)
 			
+	def _on_button_release_event(self, widget, event):
+		if self.__displayed_context_menu:
+			self.__displayed_context_menu = False
+			return
+		if event.button==1:
+			self.emit('feed-clicked')
+			
 	def _get_context_menu(self, is_filter):
 		menu = gtk.Menu()
 		
@@ -1092,6 +1131,9 @@ class FeedList(gobject.GObject):
 			menu.append(item)
 		
 		menu.show_all()
+		def realized(o):
+			self.__displayed_context_menu = True
+		menu.connect('realize', realized)
 		return menu
 
 	def do_context_menu(self, widget, event):
@@ -1100,7 +1142,7 @@ class FeedList(gobject.GObject):
 		if path is None: #nothing selected
 			return
 		selected = model[path[0]][FEEDID]
-		is_filter = self._db.is_feed_filter(selected)  
+		is_filter = self._app.db.is_feed_filter(selected)  
 
 		menu = self._get_context_menu(is_filter)
 
@@ -1123,7 +1165,7 @@ class FeedList(gobject.GObject):
 		if utils.RUNNING_SUGAR:
 			title='<span size="x-small">'+title+'</span>'
 		elif utils.RUNNING_HILDON:
-			title='<span size="x-small">'+title+'</span>'
+			title='<span size="large">'+title+'</span>'
 		try:
 			if flag & ptvDB.F_UNVIEWED == ptvDB.F_UNVIEWED:
 					title="<b>"+utils.my_quote(title)+"</b>"
@@ -1143,10 +1185,17 @@ class FeedList(gobject.GObject):
 		if not title:
 			return _("Please wait...")
 		try:
-			if not selected:
-				title = utils.my_quote(title)+'\n<span color="#777777" size="x-small"><i>'+utils.my_quote(first_entry_title)+'</i></span>'
+			if utils.RUNNING_HILDON:
+				if not selected:
+					title = '<span size="large">'+utils.my_quote(title)+'</span>\n<span color="#777777"><i>'+utils.my_quote(first_entry_title)+'</i></span>'
+				else:
+					title = '<span size="large">'+utils.my_quote(title)+'</span>\n<span><i>'+utils.my_quote(first_entry_title)+'</i></span>'
 			else:
-				title = utils.my_quote(title)+'\n<span size="x-small"><i>'+utils.my_quote(first_entry_title)+'</i></span>'
+				if not selected:
+					title = utils.my_quote(title)+'\n<span color="#777777" size="x-small"><i>'+utils.my_quote(first_entry_title)+'</i></span>'
+				else:
+					title = utils.my_quote(title)+'\n<span size="x-small"><i>'+utils.my_quote(first_entry_title)+'</i></span>'	
+			
 			if flag & ptvDB.F_UNVIEWED == ptvDB.F_UNVIEWED:
 				if unread == 0:
 					logging.warning("Flag says there are unviewed, but count says no.  not setting bold")
