@@ -5,8 +5,6 @@ import sys, os, os.path
 import sets
 import logging
 
-import code
-
 import ptvDB
 import penguintv
 import Player
@@ -42,17 +40,18 @@ N_FEEDS     = 0
 N_PLAYER    = 1
 N_DOWNLOADS = 2
 
-#import EditTagsMultiDialog
-import TagEditorNG
-import AddSearchTagDialog
-import EditSearchesDialog
-import FeedFilterDialog
-import FeedPropertiesDialog
-import FeedFilterPropertiesDialog
-import SynchronizeDialog
-import FilterSelectorDialog
-import MainWindow, FeedList, EntryList, EntryView, PlanetView, DownloadView, EntryFormatter
+import FeedList, PlanetView, DownloadView, EntryFormatter
 
+if utils.HAS_SEARCH:
+	import FeedFilterPropertiesDialog
+	import AddSearchTagDialog
+	import EditSearchesDialog
+	import FeedFilterDialog
+
+if not utils.RUNNING_HILDON:
+	import SynchronizeDialog
+	import EntryList, EntryView
+	
 if utils.HAS_GSTREAMER:
 	import GStreamerPlayer
 
@@ -71,13 +70,12 @@ class MainWindow(gobject.GObject):
 						   ([]))
 	}
 
-	def __init__(self, app, glade_prefix, use_internal_player=False, window=None, status_icon=None):
+	def __init__(self, app, use_internal_player=False, window=None, status_icon=None):
 		gobject.GObject.__init__(self)
 		self._app = app
 		self._window_inited = False
 		self._mm = self._app.mediamanager
-		self._db = self._app.db #this and app are always in the same thread
-		self._glade_prefix = glade_prefix
+		self._glade_prefix = utils.get_glade_prefix()
 		self._widgetTree = None
 		self._menu_widgettree = None
 		self.window_maximized = False
@@ -99,7 +97,7 @@ class MainWindow(gobject.GObject):
 		self._active_filter_index = FeedList.ALL
 		self._active_filter_path = (0,)
 		
-		if not utils.RUNNING_SUGAR:
+		if not utils.RUNNING_SUGAR and not utils.RUNNING_HILDON:
 			pixbuf = gtk.gdk.pixbuf_new_from_file(utils.get_image_path('ev_online.png'))
 			source = gtk.IconSource()
 			source.set_pixbuf(pixbuf)
@@ -118,11 +116,12 @@ class MainWindow(gobject.GObject):
 		
 		##other WINDOWS we open
 		if utils.HAS_SEARCH:
-			self._window_add_search = AddSearchTagDialog.AddSearchTagDialog(gtk.glade.XML(os.path.join(self._glade_prefix,'penguintv.glade'), "window_add_search_tag",'penguintv'),self._app)
-			self._feed_filter_properties_dialog = FeedFilterPropertiesDialog.FeedFilterPropertiesDialog(gtk.glade.XML(os.path.join(self._glade_prefix,'penguintv.glade'), "window_filter_properties",'penguintv'),self._app)
+			self._window_add_search = AddSearchTagDialog.AddSearchTagDialog(gtk.glade.XML(os.path.join(self._glade_prefix,'extra_dialogs.glade'), "window_add_search_tag",'penguintv'),self._app)
+			self._feed_filter_properties_dialog = FeedFilterPropertiesDialog.FeedFilterPropertiesDialog(gtk.glade.XML(os.path.join(self._glade_prefix,'extra_dialogs.glade'), "window_filter_properties",'penguintv'),self._app)
 		if not utils.RUNNING_SUGAR and not utils.RUNNING_HILDON:
-			self._sync_dialog = SynchronizeDialog.SynchronizeDialog(os.path.join(self._glade_prefix,'penguintv.glade'), self._app)
-
+			self._sync_dialog = SynchronizeDialog.SynchronizeDialog(os.path.join(self._glade_prefix,'extra_dialogs.glade'), self._app)
+		
+		self._window_add_feed = None
 		self._filter_selector_dialog = None
 		self._feed_properties_dialog = None
 		
@@ -140,9 +139,13 @@ class MainWindow(gobject.GObject):
 		#most of the initialization is done on Show()
 		if utils.RUNNING_SUGAR:
 			gobject.idle_add(self.Show, window)
-
+	
 	def __link_activated_cb(self, o, link):
 		self._app.activate_link(link)
+		
+	def __feed_clicked_cb(self, o):
+		if utils.RUNNING_HILDON:
+			self.feed_tabs.set_current_page(1)
 				
 	def __entrylistview_list_resized_cb(self, entrylistview, new_width):
 		if self.layout == "widescreen" and self.app_window is not None:			
@@ -210,9 +213,9 @@ class MainWindow(gobject.GObject):
 	def Show(self, dock_widget = None):
 		"""shows the main window. if given a widget, it will put itself in the widget.  otherwise load a regular
 		application window"""
-		if not utils.HAS_MOZILLA and self.layout.endswith("planet"):
-			logging.warning("requested planet layout, but can't use because gtkmozembed isn't installed correctly (won't import)")
-			self.layout = "standard"
+		#if not utils.HAS_MOZILLA and self.layout.endswith("planet"):
+		#	logging.warning("requested planet layout, but can't use because gtkmozembed isn't installed correctly (won't import)")
+		#	self.layout = "standard"
 		if utils.RUNNING_SUGAR:  #if we are in OLPC mode and just have to supply a widget...
 			self._status_view = None
 			self._disk_usage_widget = None
@@ -231,7 +234,8 @@ class MainWindow(gobject.GObject):
 			
 			self._connection_button = None
 			
-			self._widgetTree = gtk.glade.XML(self._glade_prefix+'/penguintv.glade', 'toolbar_holder','penguintv')
+			#TODO fixme
+			self._widgetTree = gtk.glade.XML(os.path.join(self._glade_prefix,'..','penguintv.glade'), 'toolbar_holder','penguintv')
 			self.toolbar = self._load_sugar_toolbar()
 			self.toolbar.show()
 			
@@ -265,7 +269,7 @@ class MainWindow(gobject.GObject):
 			self._connection_button = None
 			
 			logging.debug("Hildon: getting toolbar")
-			self._widgetTree = gtk.glade.XML(self._glade_prefix+'/penguintv.glade', 'hildon_toolbar_holder','penguintv')
+			self._widgetTree = gtk.glade.XML(os.path.join(self._glade_prefix,'hildon.glade'), 'hildon_toolbar_holder','penguintv')
 			self.toolbar = self._load_toolbar()
 			self.toolbar.unparent()
 			
@@ -276,16 +280,19 @@ class MainWindow(gobject.GObject):
 			self.window.add_toolbar(self.toolbar)
 			self._h_app.add_window(self.window)
 			
-			self._menu_widgettree = gtk.glade.XML(self._glade_prefix+'/penguintv.glade', 'hildon_menu','penguintv')
+			self._menu_widgettree = gtk.glade.XML(os.path.join(self._glade_prefix,'hildon.glade'), 'hildon_menu','penguintv')
 			menu = self._build_hildon_menu(self._menu_widgettree)
 			self.window.set_menu(menu)
 			
 			show_notifs_item = self._menu_widgettree.get_widget('show_notifications')
-			show_notifs_item.set_active(self._db.get_setting(ptvDB.BOOL, 
+			show_notifs_item.set_active(self._app.db.get_setting(ptvDB.BOOL, 
 		                           '/apps/penguintv/show_notifications', True))
 			
 			self.window.show_all()
 			
+			if not utils.HAS_STATUS_ICON:
+				show_notifs_item.hide()
+			
 			for key in dir(self.__class__): #python insaneness
 				if key[:3] == 'on_':
 					self._widgetTree.signal_connect(key, getattr(self, key))
@@ -302,8 +309,10 @@ class MainWindow(gobject.GObject):
 				self._widgetTree.get_widget('separator11').hide()
 				self._widgetTree.get_widget('reindex_searches').hide()
 				self._widgetTree.get_widget('add_feed_filter').hide()
-			if not utils.HAS_MOZILLA:
-				self._widgetTree.get_widget('planet_layout').hide()
+			#if not utils.HAS_MOZILLA:
+				#self._widgetTree.get_widget('planet_layout').hide()
+			if not utils.HAS_STATUS_ICON:
+				self._widgetTree.get_widget('show_notifications').hide()
 			self.window = self.app_window
 			
 		self._notebook.show_only(N_FEEDS)
@@ -390,7 +399,7 @@ class MainWindow(gobject.GObject):
 		self._sugar_remove_button = ToolButton('gtk-remove')
 		vbox = gtk.VBox()
 		#vbox.set_size_request(300, 200)
-		label = gtk.Label(_('Really delete feed?'))
+		label = gtk.Label(_('Really remove feed?'))
 		vbox.pack_start(label)
 		hbox = gtk.HBox()
 		expander_label = gtk.Label(' ')
@@ -435,7 +444,10 @@ class MainWindow(gobject.GObject):
 		from sugar.graphics.toolbutton import ToolButton
 		from sugar.graphics.palette import Palette
 		
-		content = self._app.window_add_feed.extract_content()
+		if self._window_add_feed is None:
+			self.show_window_add_feed()
+			self.hide_window_add_feed()
+		content = self._window_add_feed.extract_content()
 		palette = Palette(_('Add Feed'))
 		palette.set_content(content)
 		self.sugar_add_button.set_palette(palette)
@@ -479,17 +491,17 @@ class MainWindow(gobject.GObject):
 			return self._progress.get_fraction()
 			
 	def _load_app_window(self):
-		self._widgetTree = gtk.glade.XML(self._glade_prefix+'/penguintv.glade', 'app','penguintv')
+		self._widgetTree = gtk.glade.XML(os.path.join(self._glade_prefix,'desktop.glade'), 'app','penguintv')
 		self._menu_widgettree = self._widgetTree
 		
 		notebook_dock = self._widgetTree.get_widget('layout_dock')
 		self.app_window = self._widgetTree.get_widget('app')
 		
 		fancy_feedlist_item = self._menu_widgettree.get_widget('fancy_feed_display')
-		fancy_feedlist_item.set_active(self._db.get_setting(ptvDB.BOOL, 
+		fancy_feedlist_item.set_active(self._app.db.get_setting(ptvDB.BOOL, 
 		                               '/apps/penguintv/fancy_feedlist', True))
 		show_notifs_item = self._menu_widgettree.get_widget('show_notifications')
-		show_notifs_item.set_active(self._db.get_setting(ptvDB.BOOL, 
+		show_notifs_item.set_active(self._app.db.get_setting(ptvDB.BOOL, 
 		                           '/apps/penguintv/show_notifications', True))
 		self._widgetTree.get_widget(self.layout+"_layout").set_active(True)
 		self.app_window.set_icon_from_file(utils.get_image_path('penguintvicon.png'))
@@ -511,13 +523,13 @@ class MainWindow(gobject.GObject):
 		self._layout_dock.add(self.load_layout())
 
 		#sizing for the window comes from gconf
-		x = self._db.get_setting(ptvDB.INT, '/apps/penguintv/app_window_position_x', 40)
-		y = self._db.get_setting(ptvDB.INT, '/apps/penguintv/app_window_position_y', 40)
+		x = self._app.db.get_setting(ptvDB.INT, '/apps/penguintv/app_window_position_x', 40)
+		y = self._app.db.get_setting(ptvDB.INT, '/apps/penguintv/app_window_position_y', 40)
 		if x < 0: x = 0
 		if y < 0: y = 0
 		self.app_window.move(x,y)
-		w = self._db.get_setting(ptvDB.INT, '/apps/penguintv/app_window_size_x', 800)
-		h = self._db.get_setting(ptvDB.INT, '/apps/penguintv/app_window_size_y', 500)
+		w = self._app.db.get_setting(ptvDB.INT, '/apps/penguintv/app_window_size_x', 800)
+		h = self._app.db.get_setting(ptvDB.INT, '/apps/penguintv/app_window_size_y', 500)
 		
 		def do_maximize():
 			self.app_window.maximize()
@@ -532,10 +544,8 @@ class MainWindow(gobject.GObject):
 
 		if self.layout.endswith("planet"):
 			self._menu_widgettree.get_widget('entry_menu_item').hide()
-			self._menu_widgettree.get_widget('showkept_cb').show()
 		else:
 			self._menu_widgettree.get_widget('entry_menu_item').show()
-			self._menu_widgettree.get_widget('showkept_cb').hide()
 			
 		self.app_window.show_all()
 			
@@ -543,11 +553,33 @@ class MainWindow(gobject.GObject):
 			if key[:3] == 'on_':
 				self._widgetTree.signal_connect(key, getattr(self, key))
 				
+	def show_window_add_feed(self, autolocation=True):
+		import AddFeedDialog
+		if self._window_add_feed is None:
+			if utils.RUNNING_HILDON:
+				self._window_add_feed = AddFeedDialog.AddFeedDialog(gtk.glade.XML(os.path.join(self._glade_prefix,'hildon_dialog_add_feed.glade'), "window_add_feed",'penguintv'),self._app) #MAGIC
+			else:
+				self._window_add_feed = AddFeedDialog.AddFeedDialog(gtk.glade.XML(os.path.join(self._glade_prefix,'dialog_add_feed.glade'), "window_add_feed",'penguintv'),self._app) #MAGIC
+		self._window_add_feed.show(autolocation)
+			
+	def hide_window_add_feed(self):
+		if self._window_add_feed is None:
+			self.show_window_add_feed()
+			
+		self._window_add_feed.hide()
+		
+	def set_window_add_feed_location(self, url):
+		assert self._window_add_feed is not None
+		self._window_add_feed.set_location(url)
+				
 	def load_notebook(self):
 		self._notebook = NotebookManager()
 		self._notebook.set_property('tab-border',0)
-		label = gtk.Label(_('<span size="small">Feeds</span>'))
-		label.set_property('use-markup',True)
+		if utils.RUNNING_HILDON:
+			label = gtk.Label(_('Feeds'))
+		else:
+			label = gtk.Label(_('<span size="small">Feeds</span>'))
+			label.set_property('use-markup',True)
 		vbox = gtk.VBox()
 		self._notebook.append_page(vbox, label)
 		
@@ -558,13 +590,19 @@ class MainWindow(gobject.GObject):
 			self._gstreamer_player.connect('items-removed', self._on_player_items_removed)
 			self._gstreamer_player.Show()
 			self.emit('player-show')
-		self._player_label = gtk.Label('<span size="small">'+_('Player')+'</span>')
-		self._player_label.set_property('use-markup',True)
+		if utils.RUNNING_HILDON:
+			self._player_label = gtk.Label(_('Player'))
+		else:
+			self._player_label = gtk.Label('<span size="small">'+_('Player')+'</span>')
+			self._player_label.set_property('use-markup',True)
 		self._notebook.append_page(p_vbox, self._player_label)
 		
-		self._downloads_label = gtk.Label(_('<span size="small">Downloads</span>'))
-		self._downloads_label.set_property('use-markup',True)
-		self._download_view = DownloadView.DownloadView(self._app, self._mm, self._db, self._glade_prefix+'/penguintv.glade')
+		if utils.RUNNING_HILDON:
+			self._downloads_label = gtk.Label(_('Downloads'))
+		else:
+			self._downloads_label = gtk.Label(_('<span size="small">Downloads</span>'))
+			self._downloads_label.set_property('use-markup',True)
+		self._download_view = DownloadView.DownloadView(self._app, self._mm, self._app.db, os.path.join(self._glade_prefix,'dialogs.glade'))
 		self._notebook.append_page(self._download_view.get_widget(), self._downloads_label)
 		
 		#self._notebook.set_show_tabs(False)
@@ -586,34 +624,43 @@ class MainWindow(gobject.GObject):
 		self._notebook.set_current_page(page)
 		
 	def load_layout(self):
-		components = gtk.glade.XML(os.path.join(self._glade_prefix,'penguintv.glade'), 
-		                           self.layout+'_layout_container','penguintv') #MAGIC
+		components = gtk.glade.XML(os.path.join(self._glade_prefix,self.layout+'.glade'), 
+		               self.layout+'_layout_container','penguintv') #MAGIC
 		self._layout_components = components
 		self._layout_container = components.get_widget(self.layout+'_layout_container')
 		#dock_widget.add(self._layout_container)
 		
-		fancy = self._db.get_setting(ptvDB.BOOL, '/apps/penguintv/fancy_feedlist', True)
-		if utils.RUNNING_SUGAR or utils.RUNNING_HILDON:
+		fancy = self._app.db.get_setting(ptvDB.BOOL, '/apps/penguintv/fancy_feedlist', True)
+		if utils.RUNNING_SUGAR:
 			fancy = False
+		elif utils.RUNNING_HILDON:
+			fancy = True
 		
-		self.feed_list_view = FeedList.FeedList(components,self._app, self._db, fancy)
-		assert utils.HAS_MOZILLA
-		renderer = EntryFormatter.MOZILLA
-		
-		if self.layout.endswith("planet") and renderer != EntryFormatter.MOZILLA:
-			self.layout = "standard"
-			return self.load_layout()	
+		self.feed_list_view = FeedList.FeedList(components,self._app, fancy)
+		#renderer = EntryFormatter.MOZILLA
+		#renderer = EntryFormatter.GTKHTML
+		if utils.RUNNING_HILDON:
+			renderer = EntryFormatter.GTKHTML
+		elif utils.HAS_MOZILLA:
+			renderer = EntryFormatter.MOZILLA
+		else:
+			logging.warning("Mozilla (gtkmozembed) not found, falling back to GTKHTML")
+			renderer = EntryFormatter.GTKHTML
+		#
+		#if self.layout.endswith("planet") and renderer != EntryFormatter.MOZILLA:
+		#	self.layout = "standard"
+		#	return self.load_layout()	
 		
 		if not self.layout.endswith("planet"):
 			self.entry_list_view = EntryList.EntryList(components, self._app, 
-			                                           self.feed_list_view, self, self._db)
+			                                           self.feed_list_view, self)
 			self.entry_view = EntryView.EntryView(components, self.feed_list_view, 
 										          self.entry_list_view, self._app, self, renderer)
 		else:
 			#self.entry_view = PlanetView.PlanetView(components, self.feed_list_view, 
-			#							            self._app, self, self._db, renderer)
+			#							            self._app, self, self._app.db, renderer)
 			self.entry_view = PlanetView.PlanetView(components.get_widget('html_dock'), 
-													self, self._db, self._app.glade_prefix,
+													self, self._app.db, utils.get_share_prefix(),
 													self.feed_list_view, self._app, 
 													renderer)
 			self.entry_list_view = self.entry_view
@@ -624,6 +671,7 @@ class MainWindow(gobject.GObject):
 				
 		#some more signals
 		self.feed_list_view.connect('link-activated', self.__link_activated_cb)
+		self.feed_list_view.connect('feed-clicked', self.__feed_clicked_cb)
 
 		if not self.layout.endswith("planet"):
 			self.entry_list_view.connect('entrylist-resized', self.__entrylistview_list_resized_cb)
@@ -633,7 +681,12 @@ class MainWindow(gobject.GObject):
 		self.entry_view.connect('link-activated', self.__link_activated_cb)
 				
 		#major WIDGETS
-		self.feed_pane = components.get_widget('feed_pane')
+		if not utils.RUNNING_HILDON:
+			self.feed_pane = components.get_widget('feed_pane')
+		else:
+			self.feed_tabs = components.get_widget('feed_tabs')
+			self.feed_tabs.set_current_page(0)
+			self.feed_pane = None
 		self._feedlist = components.get_widget('feedlistview')
 		if self.layout.endswith("planet"):
 			self.entry_pane = self.feed_pane #cheat
@@ -642,15 +695,27 @@ class MainWindow(gobject.GObject):
 		
 		self._filter_container = components.get_widget('filter_container')
 		self._filter_unread_checkbox = components.get_widget('unread_filter')
-		self._filter_selector_combo = components.get_widget('filter_selector_combo')
 		self._filter_tree = gtk.TreeStore(str,      #filter displayable
 										  str,		#filter name
 										  int,      #seperator
 										  bool)     #visible
+		if utils.RUNNING_HILDON:
+			eventbox = components.get_widget('filter_selector_eventbox')
+			self._filter_selector_combo = gtk.ComboBox(self._filter_tree)
+			cell = gtk.CellRendererText()
+			cell.set_property("size-points", 24)
+			self._filter_selector_combo.pack_start(cell, True)
+			self._filter_selector_combo.add_attribute(cell, 'text', 0)
+			self._filter_selector_combo.connect('changed', self.on_filter_changed)
+			eventbox.add(self._filter_selector_combo)
+		else:
+			self._filter_selector_combo = components.get_widget('filter_selector_combo')
+		
 										  
 		filter_filter = self._filter_tree.filter_new()
 		filter_filter.set_visible_column(3)
-										  
+		
+		#if not utils.RUNNING_HILDON:								  
 		self._filter_selector_combo.set_model(filter_filter)
 		self._filter_selector_combo.set_row_separator_func(lambda model,iter:model[iter][2]==1)
 		
@@ -684,22 +749,22 @@ class MainWindow(gobject.GObject):
 		self._feedlist.drag_dest_set(gtk.DEST_DEFAULT_ALL, drop_types, gtk.gdk.ACTION_COPY)
 
 		if not self.layout.endswith("planet"):
-			val = self._db.get_setting(ptvDB.INT, '/apps/penguintv/entry_pane_position', 370)
+			val = self._app.db.get_setting(ptvDB.INT, '/apps/penguintv/entry_pane_position', 370)
 			if val < 10: val = 50
 			self.entry_pane.set_position(val)
 		
-		if utils.RUNNING_HILDON:
-			f_p_default = 270
-		else:
+		if not utils.RUNNING_HILDON:
 			f_p_default = 370
-		val = self._db.get_setting(ptvDB.INT, '/apps/penguintv/feed_pane_position', f_p_default)
-		if val < 10: val=50
-		self.feed_pane.set_position(val)
+			val = self._app.db.get_setting(ptvDB.INT, '/apps/penguintv/feed_pane_position', f_p_default)
+			if val < 10: val=50
+			
+		if self.feed_pane is not None:
+			self.feed_pane.connect('realize', self._on_feed_pane_realized, val)
 		
 		if not self.changing_layout:
 			self.set_active_filter(FeedList.ALL)
 		
-			val = self._db.get_setting(ptvDB.STRING, '/apps/penguintv/default_filter')
+			val = self._app.db.get_setting(ptvDB.STRING, '/apps/penguintv/default_filter')
 			if val is not None:
 				try:
 					filter_index = [row[F_NAME] for row in self._filters].index(val)
@@ -729,7 +794,8 @@ class MainWindow(gobject.GObject):
 		del self.entry_view
 				
 		#some widgets
-		del self.feed_pane
+		if self.feed_pane is not None:
+			del self.feed_pane
 		del self._feedlist
 		if not self.layout.endswith("planet"):
 			del self.entry_pane
@@ -780,8 +846,10 @@ class MainWindow(gobject.GObject):
 			self.search_container.hide_all()
 			
 		#elif self._notebook.get_current_page() == N_FEEDS:
-		self._db.set_setting(ptvDB.INT, '/apps/penguintv/feed_pane_position', self.feed_pane.get_position())
-		self._db.set_setting(ptvDB.INT, '/apps/penguintv/entry_pane_position', self.entry_pane.get_position())
+		if self.feed_pane is not None:
+			self._app.db.set_setting(ptvDB.INT, '/apps/penguintv/feed_pane_position', self.feed_pane.get_position())
+		if self.entry_pane is not None:
+			self._app.db.set_setting(ptvDB.INT, '/apps/penguintv/entry_pane_position', self.entry_pane.get_position())
 		if not utils.RUNNING_HILDON:
 			if self.layout.endswith('planet'):
 				self.entry_pane.set_position(0)
@@ -817,10 +885,10 @@ class MainWindow(gobject.GObject):
 		#elif self._notebook.get_current_page() == N_FEEDS:
 		if not utils.RUNNING_HILDON:
 			if self.layout.endswith('planet'):
-				val = self._db.get_setting(ptvDB.INT, '/apps/penguintv/entry_pane_position', 370)
+				val = self._app.db.get_setting(ptvDB.INT, '/apps/penguintv/entry_pane_position', 370)
 				self.entry_pane.set_position(val)
 			else:
-				val = self._db.get_setting(ptvDB.INT, '/apps/penguintv/feed_pane_position', 370)
+				val = self._app.db.get_setting(ptvDB.INT, '/apps/penguintv/feed_pane_position', 370)
 				self.feed_pane.set_position(val)
 			
 		self._notebook.set_keep_hidden(False)
@@ -848,7 +916,7 @@ class MainWindow(gobject.GObject):
 			gobject.idle_add(_unfullscreen_finish)
 	
 	def on_about_activate(self,event):
-		widgets = gtk.glade.XML(os.path.join(self._glade_prefix,'penguintv.glade'), "aboutdialog1",'penguintv')
+		widgets = gtk.glade.XML(os.path.join(self._glade_prefix,'dialogs.glade'), "aboutdialog1",'penguintv')
 		about_box = widgets.get_widget('aboutdialog1')
 		about_box.set_name('PenguinTV')
 		about_box.set_version(utils.VERSION)
@@ -890,13 +958,13 @@ class MainWindow(gobject.GObject):
 			logging.warning("Please wait until feeds have loaded before adding a new one")
 			return 
 		self._notebook.set_current_page(N_FEEDS)
-		self._app.window_add_feed.show() #not modal / blocking
+		self.show_window_add_feed() #not modal / blocking
 		
 	def on_add_feed_filter_activate(self,event):
 		selected = self.feed_list_view.get_selected()
 		if selected:
-			title = self._db.get_feed_title(selected)
-			dialog = FeedFilterDialog.FeedFilterDialog(gtk.glade.XML(self._glade_prefix+'/penguintv.glade', "window_feed_filter",'penguintv'),self._app)
+			title = self._app.db.get_feed_title(selected)
+			dialog = FeedFilterDialog.FeedFilterDialog(gtk.glade.XML(os.path.join(self._glade_prefix,'extra_dialogs.glade'), "window_feed_filter",'penguintv'),self._app)
 			dialog.show()
 			dialog.set_pointed_feed(selected,title)
 			d = { 'title':title }
@@ -914,30 +982,37 @@ class MainWindow(gobject.GObject):
 	def on_connection_button_clicked(self, event):
 		self._app.toggle_net_connection()
 		
+	def pane_to_feeds(self):
+		self.feed_tabs.set_current_page(0)
+		
 	def on_feed_add_clicked(self, event):
 		if self._state == S_MAJOR_DB_OPERATION:
 			logging.warning("Please wait until feeds have loaded before adding a new one")
 			return 
-		self._app.window_add_feed.show() #not modal / blocking
+		self.show_window_add_feed() #not modal / blocking
 	
 	#def on_feed_pane_expose_event(self, widget, event):
 	#	self.feed_list_view.resize_columns(self.feed_pane.get_position())
 	
 	def on_feed_properties_activate(self, event):
+		import FeedPropertiesDialog
 		selected = self.feed_list_view.get_selected()
 		if selected:
 			#title, description, url, link
-			feed_info = self._db.get_feed_info(selected)
+			feed_info = self._app.db.get_feed_info(selected)
 			if self._feed_properties_dialog is None:
-				self._feed_properties_dialog = FeedPropertiesDialog.FeedPropertiesDialog(gtk.glade.XML(os.path.join(self._glade_prefix,'penguintv.glade'), "window_feed_properties",'penguintv'),self._app)
+				if utils.RUNNING_HILDON:
+					self._feed_properties_dialog = FeedPropertiesDialog.FeedPropertiesDialog(gtk.glade.XML(os.path.join(self._glade_prefix,'hildon_dialogs.glade'), "window_feed_properties",'penguintv'),self._app)
+				else:
+					self._feed_properties_dialog = FeedPropertiesDialog.FeedPropertiesDialog(gtk.glade.XML(os.path.join(self._glade_prefix,'dialogs.glade'), "window_feed_properties",'penguintv'),self._app)
 			self._feed_properties_dialog.set_feedid(selected)
 			self._feed_properties_dialog.set_title(feed_info['title'])
 			self._feed_properties_dialog.set_rss(feed_info['url'])
 			self._feed_properties_dialog.set_description(feed_info['description'])
 			self._feed_properties_dialog.set_link(feed_info['link'])
 			self._feed_properties_dialog.set_last_poll(feed_info['lastpoll'])
-			self._feed_properties_dialog.set_tags(self._db.get_tags_for_feed(selected))
-			self._feed_properties_dialog.set_flags(self._db.get_flags_for_feed(selected))
+			self._feed_properties_dialog.set_tags(self._app.db.get_tags_for_feed(selected))
+			self._feed_properties_dialog.set_flags(self._app.db.get_flags_for_feed(selected))
 			if self._app.feed_refresh_method == penguintv.REFRESH_AUTO:
 				self._feed_properties_dialog.set_next_poll(feed_info['lastpoll']+feed_info['pollfreq'])
 			else:
@@ -948,7 +1023,7 @@ class MainWindow(gobject.GObject):
 		selected = self.feed_list_view.get_selected()
 		if selected:
 			#title, description, url, link
-			feed_info = self._db.get_feed_info(selected)
+			feed_info = self._app.db.get_feed_info(selected)
 			self._feed_filter_properties_dialog.set_feed_id(selected)
 			self._feed_filter_properties_dialog.set_pointed_feed_id(feed_info['feed_pointer'])
 			self._feed_filter_properties_dialog.set_filter_name(feed_info['title'])
@@ -976,12 +1051,16 @@ class MainWindow(gobject.GObject):
 			
 	def on_edit_tags_for_all_activate(self, event):
 		"""Bring up mass tag creation window"""
-		window_edit_tags_multi = TagEditorNG.TagEditorNG(gtk.glade.XML(os.path.join(self._glade_prefix,'penguintv.glade'), "dialog_tag_editor_ng",'penguintv'), self._app)
+		import TagEditorNG
+		window_edit_tags_multi = TagEditorNG.TagEditorNG(gtk.glade.XML(os.path.join(self._glade_prefix,'dialogs.glade'), "dialog_tag_editor_ng",'penguintv'), self._app)
 		window_edit_tags_multi.show()
 			
 	def on_export_opml_activate(self, event):
 		self._app.export_opml()
 		
+	def _on_feed_pane_realized(self, widget, val):
+		widget.set_position(val)
+		
 	def on_feedlistview_drag_data_received(self, widget, context, x, y, selection, targetType, time):
 		widget.emit_stop_by_name('drag-data-received')
 		if targetType == self._TARGET_TYPE_TEXT:
@@ -1004,28 +1083,32 @@ class MainWindow(gobject.GObject):
 	def on_feeds_poll_clicked(self,event):
 		self._app.poll_feeds()
 		
-	def set_show_kept_menuitem(self, state):
-		self._menu_widgettree.get_widget('showkept_cb').set_active(state)
+	def set_hide_entries_menuitem(self, state):
+		self._menu_widgettree.get_widget('hide_viewed_entries_cb').set_active(state)
 		
-	def set_show_kept_visibility(self, state):
-		assert self.layout.endswith("planet")
-	
+	def set_hide_entries_visibility(self, state):
 		if state:
-			self._menu_widgettree.get_widget('showkept_cb').show()
+			self._menu_widgettree.get_widget('hide_viewed_entries_cb').show()
 		else:
-			self._menu_widgettree.get_widget('showkept_cb').hide()
+			self._menu_widgettree.get_widget('hide_viewed_entries_cb').hide()
 		
-	def on_showkept_cb_toggled(self, event):
-		assert self.layout.endswith("planet")
-		self.entry_list_view.set_show_kept(self._menu_widgettree.get_widget('showkept_cb').get_active())
+	def on_hide_entries_cb_toggled(self, event):
+		self.entry_list_view.set_hide_viewed(self._menu_widgettree.get_widget('hide_viewed_entries_cb').get_active())
+		
+	def on_hide_feeds_cb_toggled(self, checkbox):
+		status = checkbox.get_active()
+		self.feed_list_view.set_unread_toggle(status)
+		self._menu_widgettree.get_widget('hide_viewed_feeds_cb').set_active(status)
+		self._filter_unread_checkbox.set_active(status)
 		
 	def on_synchronize_button_clicked(self,event):
 		self._sync_dialog.hide()
 		self._sync_dialog.on_sync_button_clicked(event)	
 				
 	def on_edit_favorite_tags(self, o=None):
+		import FilterSelectorDialog
 		if self._filter_selector_dialog is None:
-			self._filter_selector_dialog = FilterSelectorDialog.FilterSelectorDialog(gtk.glade.XML(os.path.join(self._glade_prefix,'penguintv.glade'), "dialog_tag_favorites",'penguintv'),self)
+			self._filter_selector_dialog = FilterSelectorDialog.FilterSelectorDialog(gtk.glade.XML(os.path.join(self._glade_prefix,'dialogs.glade'), "dialog_tag_favorites",'penguintv'),self)
 		self._filter_selector_dialog.set_taglists(self._filters, self._favorite_filters)
 		self._filter_selector_dialog.Show()
 	
@@ -1054,12 +1137,17 @@ class MainWindow(gobject.GObject):
 			self._active_filter_index = index
 			self._active_filter_path = model.get_path(it)
 			
-			if utils.HAS_SEARCH:
-				if index == FeedList.SEARCH:
-					self._filter_tree[FeedList.SEARCH][3] = True
-				else:
-					self._filter_tree[FeedList.SEARCH][3] = False
-				model.refilter()
+			if utils.HAS_SEARCH and index == FeedList.SEARCH:
+				self._filter_tree[FeedList.SEARCH][3] = True
+			else:
+				self._filter_tree[FeedList.SEARCH][3] = False
+				
+			if utils.HAS_STATUS_ICON:
+				self._filter_tree[FeedList.NOTIFY][3] = True
+			else:
+				self._filter_tree[FeedList.NOTIFY][3] = False
+
+			model.refilter()
 
 		self._activate_filter()
 		
@@ -1082,12 +1170,17 @@ class MainWindow(gobject.GObject):
 		
 	def set_active_filter(self, index):
 		model = self._filter_selector_combo.get_model()
-		if utils.HAS_SEARCH:
-			if index == FeedList.SEARCH:
-				self._filter_tree[FeedList.SEARCH][3] = True
-			else:
-				self._filter_tree[FeedList.SEARCH][3] = False
-			model.refilter()
+		if utils.HAS_SEARCH and index == FeedList.SEARCH:
+			self._filter_tree[FeedList.SEARCH][3] = True
+		else:
+			self._filter_tree[FeedList.SEARCH][3] = False
+		
+		if utils.HAS_STATUS_ICON:
+			self._filter_tree[FeedList.NOTIFY][3] = True
+		else:
+			self._filter_tree[FeedList.NOTIFY][3] = False
+			
+		model.refilter()
 
 		self._find_path(index)
 		it = model.get_iter(self._active_filter_path)
@@ -1175,6 +1268,8 @@ class MainWindow(gobject.GObject):
 			elif keyname == 'F8':
 				if self._gstreamer_player is not None:
 					self._gstreamer_player.vol_down()
+			elif keyname == 'Escape':
+				self.feed_tabs.set_current_page(0)
 		else: #regular desktop version..
 			if keyname == 'F11':
 				self.toggle_fullscreen()
@@ -1204,7 +1299,7 @@ class MainWindow(gobject.GObject):
 		entry = self.entry_list_view.get_selected()['entry_id']
 		self._app.activate_link("unkeep:%i" % (entry,))
 		
-	def on_mark_feed_as_viewed_activate(self,event):
+	def on_mark_feed_as_viewed_activate(self, button=None, event=None):
 		feed = self.feed_list_view.get_selected()
 		if feed:
 			self._app.mark_feed_as_viewed(feed)
@@ -1255,7 +1350,10 @@ class MainWindow(gobject.GObject):
 		#		player.play()
 		#	except:
 		#		pass #fails while loading
-		self._player_label.set_markup(_('<span size="small">Player (%d)</span>') % player.get_queue_count())
+		if utils.RUNNING_HILDON:
+			self._player_label.set_markup(_('Player (%d)') % player.get_queue_count())
+		else:
+			self._player_label.set_markup(_('<span size="small">Player (%d)</span>') % player.get_queue_count())
 		#if self._state != S_MAJOR_DB_OPERATION:
 		#	tip = tooltips(self._player_label)
 		#	tip.display_notification("title", "text")
@@ -1265,7 +1363,10 @@ class MainWindow(gobject.GObject):
 			self._notebook.hide_page(N_PLAYER)
 			self.emit('player-hide')
 			player.stop()
-		self._player_label.set_markup(_('<span size="small">Player (%d)</span>') % player.get_queue_count())
+		if utils.RUNNING_HILDON:
+			self._player_label.set_markup(_('Player (%d)') % player.get_queue_count())
+		else:
+			self._player_label.set_markup(_('<span size="small">Player (%d)</span>') % player.get_queue_count())
 		
 	def on_preferences_activate(self, event):
 		self._app.window_preferences.show()
@@ -1288,7 +1389,7 @@ class MainWindow(gobject.GObject):
 		
 	def on_refresh_visible_feeds_activate(self, event):
 		if self._active_filter_index > FeedList.SEARCH:
-			feeds = self._db.get_feeds_for_tag(self._active_filter_name)
+			feeds = self._app.db.get_feeds_for_tag(self._active_filter_name)
 			self._app.do_poll_multiple(None, ptvDB.A_IGNORE_ETAG, feeds, 
 					message=_("Refreshing %s..." % self._active_filter_name))
 		elif utils.RUNNING_HILDON:
@@ -1311,8 +1412,8 @@ class MainWindow(gobject.GObject):
 		if selected:
 			self._notebook.set_current_page(N_FEEDS)
 			if not override:
-				dialog = gtk.Dialog(title=_("Really Delete Feed?"), parent=None, flags=gtk.DIALOG_MODAL, buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT, gtk.STOCK_DELETE, gtk.RESPONSE_ACCEPT))
-				label = gtk.Label(_("Are you sure you want to delete this feed, all its entries, and all its media?  \nThis operation cannot be undone."))
+				dialog = gtk.Dialog(title=_("Really Remove Feed?"), parent=None, flags=gtk.DIALOG_MODAL, buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT, gtk.STOCK_REMOVE, gtk.RESPONSE_ACCEPT))
+				label = gtk.Label(_("Are you sure you want to remove this feed, all its entries, and all its media?  \nThis operation cannot be undone."))
 				dialog.vbox.pack_start(label, True, True, 0)
 				label.show()
 				dialog.set_transient_for(self._app.main_window.get_parent())
@@ -1337,7 +1438,7 @@ class MainWindow(gobject.GObject):
 		self._app.set_state(penguintv.DEFAULT)
 		
 	def on_saved_searches_activate(self, event):
-		window_edit_saved_searches = EditSearchesDialog.EditSearchesDialog(os.path.join(self._glade_prefix,'penguintv.glade'),self._app)
+		window_edit_saved_searches = EditSearchesDialog.EditSearchesDialog(os.path.join(self._glade_prefix,'extra_dialogs.glade'),self._app)
 		window_edit_saved_searches.show()
 		del window_edit_saved_searches
 		
@@ -1377,10 +1478,10 @@ class MainWindow(gobject.GObject):
 		
 	def on_fancy_feed_display_activate(self, menuitem):
 		self.feed_list_view.set_fancy(menuitem.get_active())
-		self._db.set_setting(ptvDB.BOOL, '/apps/penguintv/fancy_feedlist', menuitem.get_active())
+		self._app.db.set_setting(ptvDB.BOOL, '/apps/penguintv/fancy_feedlist', menuitem.get_active())
 		
 	def on_show_notifications_activate(self, menuitem):
-		self._db.set_setting(ptvDB.BOOL, '/apps/penguintv/show_notifications',
+		self._app.db.set_setting(ptvDB.BOOL, '/apps/penguintv/show_notifications',
 							 menuitem.get_active())
 				
 	def activate_layout(self, layout):
@@ -1392,13 +1493,11 @@ class MainWindow(gobject.GObject):
 		self._layout_dock.remove(self._layout_container)
 		
 		self._layout_dock.add(self.load_layout())
-		self.entry_view.post_show_init()#**#
+		self.entry_view.post_show_init()
 		if self.layout.endswith("planet"):
 			self._menu_widgettree.get_widget('entry_menu_item').hide()
-			self._menu_widgettree.get_widget('showkept_cb').show()
 		else:
 			self._menu_widgettree.get_widget('entry_menu_item').show()
-			self._menu_widgettree.get_widget('showkept_cb').hide()
 
 		self._notebook.show_only(N_FEEDS)
 		if not utils.HAS_SEARCH:
@@ -1417,9 +1516,6 @@ class MainWindow(gobject.GObject):
 	def is_changing_layout(self):
 		return self.changing_layout
 		
-	def on_unread_filter_toggled(self, event): 	 
-		self.feed_list_view.set_unread_toggle(self._filter_unread_checkbox.get_active()) 	 
-		
 	def display_status_message(self, m, update_category=U_STANDARD):
 		"""displays a status message on the main status bar.  If this is a polling update or download
 		   update, we don't overwrite what's there."""	
@@ -1528,7 +1624,7 @@ class MainWindow(gobject.GObject):
 		i=0 #we set i here so that searches and regular tags have incrementing ids
 		
 		builtin = _("All Feeds")
-		text = builtin+" ("+str(len(self._db.get_feedlist()))+")"
+		text = builtin+" ("+str(len(self._app.db.get_feedlist()))+")"
 		self._filters.append([0,builtin,text,ptvDB.T_BUILTIN])
 		self._filter_tree.append(None, [text, builtin, 0, True])
 		i += 1
@@ -1539,36 +1635,37 @@ class MainWindow(gobject.GObject):
 		i += 1
 		
 		builtin = _("Notifying Feeds")
-		text = builtin+" ("+str(len(self._db.get_feeds_for_flag(ptvDB.FF_NOTIFYUPDATES)))+")"
+		text = builtin+" ("+str(len(self._app.db.get_feeds_for_flag(ptvDB.FF_NOTIFYUPDATES)))+")"
 		self._filters.append([0,builtin,text,ptvDB.T_BUILTIN])
-		self._filter_tree.append(None, [text, builtin, 0, True])
+		self._filter_tree.append(None, [text, builtin, 0, utils.HAS_STATUS_ICON])
+		i += 1
+		
+		builtin = _("Search Results")
+		self._filters.append([0,builtin,builtin,ptvDB.T_BUILTIN])
+		self._search_iter = self._filter_tree.append(None, [builtin, builtin, 0, False])
 		i += 1
 		
 		has_search = False
-		if utils.HAS_SEARCH:	
-			builtin = _("Search Results")
-			self._filters.append([0,builtin,builtin,ptvDB.T_BUILTIN])
-			self._search_iter = self._filter_tree.append(None, [builtin, builtin, 0, False])
-	
-			tags = self._db.get_all_tags(ptvDB.T_SEARCH)	
+		if utils.HAS_SEARCH:
+			tags = self._app.db.get_all_tags(ptvDB.T_SEARCH)	
 			if tags:
 				has_search = True
 				for tag,favorite in tags:
 					i+=1
 					self._filters.append([favorite, tag,tag,ptvDB.T_SEARCH])
-					completion_model.append([tag,_('tag: %s') % (tag,), i])
+					completion_model.append([tag,_('tag: %s') % (tag,), i - 1])
 					if favorite > 0:
 						self._favorite_filters.append([favorite, tag,tag, i])
 		
-		tags = self._db.get_all_tags(ptvDB.T_TAG)
+		tags = self._app.db.get_all_tags(ptvDB.T_TAG)
 		if tags:
 			self._filter_tree.append(None, ["", "", 1, True])
 			for tag,favorite in tags:
 				i+=1
-				self._filters.append([favorite, tag,tag+" ("+str(self._db.get_count_for_tag(tag))+")",ptvDB.T_TAG])
-				completion_model.append([tag,_('tag: %s') % (tag,), i])
+				self._filters.append([favorite, tag,tag+" ("+str(self._app.db.get_count_for_tag(tag))+")",ptvDB.T_TAG])
+				completion_model.append([tag,_('tag: %s') % (tag,), i - 1])
 				if favorite > 0:
-					self._favorite_filters.append([favorite, tag,tag+" ("+str(self._db.get_count_for_tag(tag))+")", i])
+					self._favorite_filters.append([favorite, tag,tag+" ("+str(self._app.db.get_count_for_tag(tag))+")", i])
 				
 		self._favorite_filters.sort()
 		self._favorite_filters = [f[1:] for f in self._favorite_filters]
@@ -1609,7 +1706,7 @@ class MainWindow(gobject.GObject):
 			i+=1
 			print t, i
 			if t != old_order[i-1]:
-				self._db.set_tag_favorite(t, i)
+				self._app.db.set_tag_favorite(t, i)
 		
 		if len(old_order) > 0:
 			i = len(old_order)-1
@@ -1618,13 +1715,13 @@ class MainWindow(gobject.GObject):
 		for t in tag_list[len(old_order):]:
 			print t, i
 			i+=1
-			self._db.set_tag_favorite(t, i)
+			self._app.db.set_tag_favorite(t, i)
 				
 		old = sets.Set(old_order)
 		new = sets.Set(tag_list)
 		removed = list(old.difference(new))
 		for t in removed:
-			self._db.set_tag_favorite(t, 0)
+			self._app.db.set_tag_favorite(t, 0)
 		self.update_filters()
 		
 	def _on_completion_match_selected(self, completion, model, iter, column):
@@ -1661,7 +1758,7 @@ class MainWindow(gobject.GObject):
 	def select_feed(self, feed_id):
 		#if we have a tag, pick the first one (really used just when adding
 		#feeds)
-		tags = self._db.get_tags_for_feed(feed_id)
+		tags = self._app.db.get_tags_for_feed(feed_id)
 		if len(tags) > 0:
 			if not self._active_filter_name in tags:
 				self.set_active_filter(FeedList.ALL)
@@ -1717,7 +1814,10 @@ class MainWindow(gobject.GObject):
 		if number == 0:
 			self._notebook.hide_page(N_DOWNLOADS)
 		else:
-			self._downloads_label.set_markup(_('<span size="small">Downloads (%d)</span>') % number)
+			if utils.RUNNING_HILDON:
+				self._downloads_label.set_markup(_('Downloads (%d)') % number)
+			else:
+				self._downloads_label.set_markup(_('<span size="small">Downloads (%d)</span>') % number)
 			self._notebook.show_page(N_DOWNLOADS)
 				
 	def desensitize(self):
