@@ -7,6 +7,9 @@
 
 import logging
 import os, os.path
+import threading
+import re
+import time
 
 import gobject
 import gtk
@@ -14,15 +17,20 @@ import gtk
 import EntryFormatter
 import ptvDB
 import utils
+import ThreadPool
 
-if not utils.RUNNING_SUGAR:
+if utils.RUNNING_HILDON:
+	import hildon
+elif utils.RUNNING_SUGAR:
+	import hulahop
+else:
 	try:
 		import gtkmozembed
 	except:
-		pass
-else:
-	import hulahop
-	
+		try:
+			from ptvmozembed import gtkmozembed
+		except:
+			pass
 
 if utils.RUNNING_HILDON:
 	ENTRIES_PER_PAGE = 5
@@ -33,6 +41,8 @@ else:
 S_DEFAULT=0
 S_SEARCH=1
 
+IMG_REGEX = re.compile("<img.*?src=[\",\'](.*?)[\",\'].*?>", re.IGNORECASE|re.DOTALL)
+
 class PlanetView(gobject.GObject):
 	"""PlanetView implementes the api for entrylist and entryview, so that the main program doesn't
 	need to know that the two objects are actually the same"""
@@ -57,7 +67,7 @@ class PlanetView(gobject.GObject):
                            [])
     }	                       
 	
-	def __init__(self, dock_widget, main_window, db, glade_path, feed_list_view=None, app=None, renderer=EntryFormatter.MOZILLA):
+	def __init__(self, dock_widget, main_window, db, share_path, feed_list_view=None, app=None, renderer=EntryFormatter.MOZILLA):
 		gobject.GObject.__init__(self)
 		#public
 		self.presently_selecting = False
@@ -74,14 +84,13 @@ class PlanetView(gobject.GObject):
 		self._renderer = renderer
 		self._css = ""
 		self._current_feed_id = -1
-		self._moz_realized = False
 		self._feed_title=""
 		self._state = S_DEFAULT
 		self._auth_info = (-1, "","") #user:pass, url
 		self._custom_message = ""
 		self._search_query = None
 		self._filter_feed = None
-		self._show_kept = False
+		self._hide_viewed = False
 		self._ignore_next_event = False
 		self._USING_AJAX = False
 		
@@ -93,7 +102,32 @@ class PlanetView(gobject.GObject):
 		
 		self._html_dock = dock_widget
 		self._scrolled_window = gtk.ScrolledWindow()
-		self._html_dock.add(self._scrolled_window)
+		if utils.RUNNING_HILDON:
+			hildon.hildon_helper_set_thumb_scrollbar(self._scrolled_window, True)
+			vbox = gtk.VBox()
+			#hbox = gtk.HBox()
+			#b = gtk.Button(stock=gtk.STOCK_GO_BACK)
+			#b.connect("clicked", self._do_pane_back)
+			#hbox.set_property("height-request", 40)
+		#	hbox.pack_start(b, False, False)
+		#	l = gtk.Label("")
+		#	hbox.pack_start(l, True, True)
+		#	vbox.pack_start(hbox, False, False)
+			vbox.pack_start(self._scrolled_window, True, True)
+			hbox = gtk.HBox()
+			self._back_button = gtk.Button(label=_("Newer Entries"))
+			self._back_button.connect("clicked", self._do_planet_up)
+			hbox.pack_start(self._back_button, False, False)
+			l = gtk.Label("")
+			hbox.pack_start(l, True, True)
+			self._forward_button = gtk.Button(label=_("Older Entries"))
+			self._forward_button.connect("clicked", self._do_planet_down)
+			hbox.pack_start(self._forward_button, False, False)
+			hbox.set_property("height-request", 36)
+			vbox.pack_start(hbox, False, False)
+			self._html_dock.add(vbox)
+		else:
+			self._html_dock.add(self._scrolled_window)
 		self._scrolled_window.set_property("hscrollbar-policy",gtk.POLICY_AUTOMATIC)
 		self._scrolled_window.set_property("vscrollbar-policy",gtk.POLICY_AUTOMATIC)
 		self._scrolled_window.set_flags(self._scrolled_window.flags() & gtk.CAN_FOCUS) 
@@ -115,17 +149,29 @@ class PlanetView(gobject.GObject):
                 style.base[gtk.STATE_INSENSITIVE].green / 256)
 		
 		if self._renderer == EntryFormatter.MOZILLA:
+			self._moz_realized = False
 			if utils.RUNNING_SUGAR:
-				f = open(os.path.join(glade_path, "mozilla-planet-olpc.css"))
+				f = open(os.path.join(share_path, "mozilla-planet-olpc.css"))
 				for l in f.readlines(): self._css += l
 				f.close()
 			else:
 				if utils.RUNNING_HILDON:
-					f = open(os.path.join(glade_path, "mozilla-planet-hildon.css"))
+					f = open(os.path.join(share_path, "mozilla-planet-hildon.css"))
 				else:
-					f = open(os.path.join(glade_path, "mozilla-planet.css"))
+					f = open(os.path.join(share_path, "mozilla-planet.css"))
 				for l in f.readlines(): self._css += l
 				f.close()
+		elif self._renderer == EntryFormatter.GTKHTML:
+			f = open(os.path.join(share_path, "gtkhtml.css"))
+			#f = open(os.path.join(share_path, "mozilla-planet-hildon.css"))
+			for l in f.readlines(): self._css += l
+			f.close()
+			self._current_scroll_v = self._scrolled_window.get_vadjustment().get_value()
+			self._current_scroll_h = self._scrolled_window.get_hadjustment().get_value()
+			self._image_pool = ThreadPool.ThreadPool(5, "PlanetView")
+			#self._image_lock = threading.Lock()
+			self._dl_total = 0
+			self._dl_count = 0
 				
 		#signals
 		self._handlers = []
@@ -153,6 +199,8 @@ class PlanetView(gobject.GObject):
 			self._handlers.append((self._app.disconnect, h_id))
 			h_id = self._app.connect('entries-unviewed', self.__entries_updated_cb)
 			self._handlers.append((self._app.disconnect, h_id))
+			h_id = app.connect('new-database', self.__new_database_cb)
+			self._handlers.append((app.disconnect, h_id))
 		screen = gtk.gdk.screen_get_default()
 		h_id = screen.connect('size-changed', self.__size_changed_cb)
 		self._handlers.append((screen.disconnect, h_id))
@@ -197,18 +245,42 @@ class PlanetView(gobject.GObject):
 				self._conf = gconf.client_get_default()
 				self._conf.notify_add('/desktop/gnome/interface/font_name',self._gconf_reset_moz_font)
 			self._reset_moz_font()
+		elif self._renderer == EntryFormatter.GTKHTML:
+			import gtkhtml2
+			import SimpleImageCache
+			import threading
+			self._scrolled_window.set_property("shadow-type",gtk.SHADOW_IN)
+			htmlview = gtkhtml2.View()
+			self._document = gtkhtml2.Document()
+			self._document.connect("link-clicked", self._gtkhtml_link_clicked)
+			htmlview.connect("on_url", self._gtkhtml_on_url)
+			self._document.connect("request-url", self._gtkhtml_request_url)
+			htmlview.get_vadjustment().set_value(0)
+			htmlview.get_hadjustment().set_value(0)
+			self._scrolled_window.set_hadjustment(htmlview.get_hadjustment())
+			self._scrolled_window.set_vadjustment(htmlview.get_vadjustment())
+			
+			self._document.clear()
+			htmlview.set_document(self._document)
+			self._scrolled_window.add(htmlview)
+			self._htmlview = htmlview
+			self._document_lock = threading.Lock()
+			self._image_cache = SimpleImageCache.SimpleImageCache()
+			
 		self.display_item()
 		self._html_dock.show_all()
 		if self._USING_AJAX:
 			logging.info("initializing ajax server")
 			import threading
 			from ajax import EntryInfoServer, MyTCPServer
+			
+			store_location = os.path.join(self._db.get_setting(ptvDB.STRING, '/apps/penguintv/media_storage_location', ""), os.path.join(utils.get_home(), "media", "images"))
 
 			while True:
 				try:
 					if PlanetView.PORT == 8050:
 						break
-					self._update_server = MyTCPServer.MyTCPServer(('', PlanetView.PORT), EntryInfoServer.EntryInfoServer)
+					self._update_server = MyTCPServer.MyTCPServer(('', PlanetView.PORT), EntryInfoServer.EntryInfoServer, store_location)
 					break
 				except:
 					PlanetView.PORT += 1
@@ -217,14 +289,14 @@ class PlanetView(gobject.GObject):
 			t = threading.Thread(None, self._update_server.serve_forever, name="PTV AJAX Server Thread")
 			t.setDaemon(True)
 			t.start()
-			img_url = "http://localhost:"+str(PlanetView.PORT)+"/"+self._update_server.get_key()
-			self._entry_formatter = EntryFormatter.EntryFormatter(self._mm, False, True, ajax_url=img_url)
-			self._search_formatter = EntryFormatter.EntryFormatter(self._mm, True, True, ajax_url=img_url)
+			self._ajax_url = "http://localhost:"+str(PlanetView.PORT)+"/"+self._update_server.get_key()
+			self._entry_formatter = EntryFormatter.EntryFormatter(self._mm, False, True, ajax_url=self._ajax_url, renderer=self._renderer)
+			self._search_formatter = EntryFormatter.EntryFormatter(self._mm, True, True, ajax_url=self._ajax_url, renderer=self._renderer)
 		else:
 			logging.info("not using ajax")
-			img_url = None
-			self._entry_formatter = EntryFormatter.EntryFormatter(self._mm, False, True, basic_progress=True)
-			self._search_formatter = EntryFormatter.EntryFormatter(self._mm, True, True, basic_progress=True)
+			self._ajax_url = None
+			self._entry_formatter = EntryFormatter.EntryFormatter(self._mm, False, True, basic_progress=True, renderer=self._renderer)
+			self._search_formatter = EntryFormatter.EntryFormatter(self._mm, True, True, basic_progress=True, renderer=self._renderer)
 		
 	def set_entry_view(self, entry_view):
 		pass
@@ -250,7 +322,8 @@ class PlanetView(gobject.GObject):
 			self.populate_entries(feed_id)
 			
 	def __feed_polled_cb(self, app, feed_id, update_data):
-		if feed_id == self._current_feed_id:
+		f_list = self._db.get_associated_feeds(feed_id)
+		if self._current_feed_id in f_list:
 			self.populate_entries(feed_id)
 			if update_data['pollfail']:
 				self.display_custom_entry("<b>"+_("There was an error trying to poll this feed.")+"</b>")
@@ -262,12 +335,14 @@ class PlanetView(gobject.GObject):
 		
 	def __entry_updated_cb(self, app, entry_id, feed_id):
 		self.update_entry_list(entry_id)
-		if feed_id == self._current_feed_id and not self._USING_AJAX:
+		f_list = self._db.get_associated_feeds(feed_id)
+		if self._current_feed_id in f_list and not self._USING_AJAX:
 			self._render_entries(mark_read=False, force=True)
 			
 	def __entries_updated_cb(self, app, viewlist):
 		for feed_id, idlist in viewlist:
-			if feed_id == self._current_feed_id:
+			f_list = self._db.get_associated_feeds(feed_id)
+			if self._current_feed_id in f_list:
 				self.populate_entries(feed_id)
 			
 	def __render_ops_updated_cb(self, app):
@@ -279,6 +354,9 @@ class PlanetView(gobject.GObject):
 		"""Redraw after xrandr calls"""
 		self._render_entries()
 		
+	def __new_database_cb(self, app, db):
+		self._db = db
+		
 	def grab_focus(self):
 		if utils.RUNNING_SUGAR:
 			self._moz.grab_focus()
@@ -325,11 +403,13 @@ class PlanetView(gobject.GObject):
 		new_feed = False
 		if feed_id != self._current_feed_id:
 			new_feed = True
-			self._show_kept = False
-			self._main_window.set_show_kept_menuitem(self._show_kept)
-			self._main_window.set_show_kept_visibility(True)
+			#self._hide_viewed = False
+			#self._main_window.set_hide_entries_menuitem(self._hide_viewed)
+			#self._main_window.set_hide_entries_visibility(True)
 			self._current_feed_id = feed_id
 			self._first_entry = 0
+			if self._renderer == EntryFormatter.GTKHTML:
+				self._gtkhtml_reset_image_dl()
 			self._entry_store={}
 			feed_info = self._db.get_feed_info(feed_id)
 			if feed_info['auth_feed']:
@@ -374,9 +454,9 @@ class PlanetView(gobject.GObject):
 		self._entrylist = [(e[0],e[3]) for e in entries]
 		self._convert_newlines = False
 		self._current_feed_id = -1
-		self._show_kept = False
-		self._main_window.set_show_kept_menuitem(self._show_kept)
-		self._main_window.set_show_kept_visibility(False)
+		self._hide_viewed = False
+		self._main_window.set_hide_entries_menuitem(self._hide_viewed)
+		self._main_window.set_hide_entries_visibility(False)
 		query = query.replace("*","")
 		self._search_query = query
 		try:
@@ -398,6 +478,8 @@ class PlanetView(gobject.GObject):
 		self._entry_store={}
 		self._entrylist = []
 		self._convert_newlines = False
+		if self._renderer == EntryFormatter.GTKHTML:
+			self._gtkhtml_reset_image_dl()
 		self._render("<html><body></body></html")
 		if self._USING_AJAX:
 			self._update_server.clear_updates()
@@ -462,8 +544,11 @@ class PlanetView(gobject.GObject):
 			except:
 				logging.error('error closing planetview server')
 		self._render("<html><body></body></html")
-		if not utils.RUNNING_SUGAR and not utils.RUNNING_HILDON:
+		if not utils.RUNNING_SUGAR and not utils.RUNNING_HILDON and self._renderer == EntryFormatter.MOZILLA:
 			gtkmozembed.pop_startup()
+		if self._renderer == EntryFormatter.GTKHTML:
+			self._image_pool.joinAll(False, False)
+			del self._image_pool
 					
 	#protected functions
 	def _render_entries(self, mark_read=False, force=False):
@@ -471,17 +556,24 @@ class PlanetView(gobject.GObject):
 
 		if self._first_entry < 0:
 			self._first_entry = 0
+			if self._renderer == EntryFormatter.GTKHTML:
+				self._gtkhtml_reset_image_dl()
 			
+		if utils.RUNNING_HILDON:
+			self._back_button.hide()
+			self._forward_button.hide()
+
 		if self._filter_feed is not None:
 			assert self._state == S_SEARCH
 			entrylist = [r for r in self._entrylist if r[1] == self._filter_feed]
 		else:
 			entrylist = self._entrylist
-			if self._show_kept:
+			if self._hide_viewed:
 				newlist = []
-				kept = self._db.get_kept_entries(self._current_feed_id)
+				#kept = self._db.get_kept_entries(self._current_feed_id)
+				unviewed = self._db.get_unread_entries(self._current_feed_id)
 				for e,f in entrylist:
-					if e in kept:
+					if e in unviewed:
 						newlist.append((e,f))
 				entrylist = newlist
 			
@@ -534,30 +626,45 @@ class PlanetView(gobject.GObject):
 		gobject.timeout_add(2000, self._do_delayed_set_viewed, self._current_feed_id, self._first_entry, self._last_entry)
 			
 		#######build HTML#######	
-		cb_status = self._show_kept and "CHECKED" or "UNCHECKED"
-		cb_function = self._show_kept and "showkept:0" or "showkept:1"
+		#cb_status = self._hide_viewed and "CHECKED" or "UNCHECKED"
+		#cb_function = self._hide_viewed and "hideviewed:0" or "hideviewed:1"
 		
 		html = []
 		html.append(self._build_header(media_exists))
 		
-		html.append(self._custom_message)
+		html.append(self._custom_message+"<br>")
 		
+		if utils.RUNNING_HILDON:
+			html.append('<a href="pane:back"><img border="0" src="file://%s"/>%s</a>' % (
+					"/usr/share/icons/hicolor/26x26/hildon/qgn_list_hw_button_esc.png",
+					_("Back to Feeds")))
+					
 		html.append("""<div id="nav_bar"><table
 					style="width: 100%; text-align: left; margin-left: auto; margin-right: auto;"
  					border="0" cellpadding="2" cellspacing="0">
 					<tbody>
 					<tr><td>""")
 		if self._first_entry > 0:
-			html.append(_('<a href="planet:up">Newer Entries</a>'))
+			if utils.RUNNING_HILDON:
+				#html.append(_('<a href="planet:up" style="font-size: 20pt">Newer Entries</a>'))
+				#html.append("""<button type="button" onclick="parent.location='planet:up'">Newer Entries</button>""")
+				self._back_button.show()
+			else:
+				html.append(_('<a href="planet:up">Newer Entries</a>'))
 		
 		html.append('</td><td style="text-align: right;">')
 		if self._last_entry < len(entrylist):
-			html.append(_('<a href="planet:down">Older Entries</a>'))
+			if utils.RUNNING_HILDON:
+				#html.append(_('<a href="planet:down" style="font-size: 20pt">Older Entries</a>'))
+				#html.append("""<button type="button" onclick="parent.location='planet:up'">Older Entries</button>""")
+				self._forward_button.show()
+			else:
+				html.append(_('<a href="planet:down">Older Entries</a>'))
 			
 		html.append("</td></tr></tbody></table></div>")
 		
 		if self._state != S_SEARCH:
-			html.append('<div class="feed_title">'+self._feed_title+"</div>")
+			html.append('<div class="feedtitle">'+self._feed_title+"</div>")
 		html += entries
 			
 		html.append("""<div id="nav_bar"><table
@@ -566,11 +673,25 @@ class PlanetView(gobject.GObject):
 					<tbody>
 					<tr><td>""")
 		if self._first_entry > 0:
-			html.append(_('<a href="planet:up">Newer Entries</a>'))
+			if utils.RUNNING_HILDON:
+				#html.append(_('<a href="planet:up" style="font-size: 20pt">Newer Entries</a>'))
+				pass
+			else:
+				html.append(_('<a href="planet:up">Newer Entries</a>'))
 		html.append('</td><td style="text-align: right;">')
 		if self._last_entry < len(entrylist):
-			html.append(_('<a href="planet:down">Older Entries</a>'))
+			if utils.RUNNING_HILDON:
+				#html.append(_('<a href="planet:down" style="font-size: 20pt">Older Entries</a>'))
+				pass
+			else:
+				html.append(_('<a href="planet:down">Older Entries</a>'))
 		html.append("</td></tr></tbody></table></div>")
+		
+		if utils.RUNNING_HILDON:
+			html.append('<a href="pane:back"><img border="0" src="file://%s"/>%s</a>' % (
+					"/usr/share/icons/hicolor/26x26/hildon/qgn_list_hw_button_esc.png",
+					_("Back to Feeds")))
+		
 		html.append("</body></html>")
 		
 		html = "".join(html)
@@ -579,18 +700,18 @@ class PlanetView(gobject.GObject):
 	def _build_header(self, media_exists):
 		if self._renderer == EntryFormatter.MOZILLA:
 			html = ["""<html><head>
-				    <meta http-equiv="Content-Type" content="text/html; charset=utf-8">
-					<style type="text/css">
-				    body { background-color: %s; color: %s; font-family: %s; font-size: %s; }
-				    %s
-				    </style>
-				    <title>title</title>""" % (self._background_color,
-											   self._foreground_color,
-											   self._moz_font, 
-											   self._moz_size, 
-											   self._css)] 
-			html.append("""<script type="text/javascript"><!--""")
+			    <meta http-equiv="Content-Type" content="text/html; charset=utf-8">
+				<style type="text/css">
+			    body { background-color: %s; color: %s; font-family: %s; font-size: %s; }
+			    %s
+			    </style>
+			    <title>title</title>""" % (self._background_color,
+										   self._foreground_color,
+										   self._moz_font, 
+										   self._moz_size, 
+										   self._css)] 
 			if self._USING_AJAX:
+				html.append("""<script type="text/javascript"><!--""")
 				html.append("""
 	            
 	            var xmlHttp
@@ -662,6 +783,14 @@ class PlanetView(gobject.GObject):
 			
 			html.append("--> </script>")
 			html.append("""</head><body><span id="errorMsg"></span><br>""")
+		else:
+			html = ["""<html><head>
+			    <meta http-equiv="Content-Type" content="text/html; charset=utf-8">
+<style type="text/css">
+%s
+</style>
+			    <title>title</title>""" % self._css] 
+			html.append("""</head><body>""")
 			
 		return "\n".join(html)
 		
@@ -677,7 +806,7 @@ class PlanetView(gobject.GObject):
 		#load the rest from db
 		if len(entry_list) > 0:
 			e_id_list = [r[0] for r in entry_list]
-			db_entries = self._db.get_entry_block(e_id_list)
+			db_entries = self._db.get_entry_block(e_id_list, self._ajax_url)
 			media = self._db.get_entry_media_block(e_id_list)
 		
 			for item in db_entries:
@@ -711,7 +840,7 @@ class PlanetView(gobject.GObject):
 		if self._entry_store.has_key(entry_id) and not force:
 			return self._entry_store[entry_id]
 		
-		item = self._db.get_entry(entry_id)
+		item = self._db.get_entry(entry_id, self._ajax_url)
 		media = self._db.get_entry_media(entry_id)
 		if media:
 			item['media']=media
@@ -735,7 +864,12 @@ class PlanetView(gobject.GObject):
 					self._entry_store[entry_id] = (new_format, item)
 					return self._entry_store[entry_id]
 			self._entry_store[entry_id] = (new_format, item)
-		index = self._entrylist.index((entry_id,item['feed_id']))
+		try:
+			index = self._entrylist.index((entry_id,self._current_feed_id))
+		except:
+			logging.warning("Told to update an entry we don't have -- can't update")
+			return self._entry_store[entry_id]
+			
 		if index >= self._first_entry and index <= self._first_entry+ENTRIES_PER_PAGE:
 			entry = self._entry_store[entry_id][1]
 			if self._USING_AJAX:
@@ -775,17 +909,107 @@ class PlanetView(gobject.GObject):
 		#logging.debug("="*80)
 		#logging.debug(html)
 		#logging.debug("="*80)
-		if self._moz_realized or utils.RUNNING_SUGAR:
-			if self._USING_AJAX:
-				self._moz.open_stream("http://localhost:"+str(PlanetView.PORT),"text/html")
+		if self._renderer == EntryFormatter.MOZILLA:
+			if self._moz_realized or utils.RUNNING_SUGAR:
+				if self._USING_AJAX:
+					self._moz.open_stream("http://localhost:"+str(PlanetView.PORT),"text/html")
+				else:
+					self._moz.open_stream("file:///","text/html")
+				while len(html)>60000:
+					part = html[0:60000]
+					html = html[60000:]
+					self._moz.append_data(part, long(len(part)))
+				self._moz.append_data(html, long(len(html)))
+				self._moz.close_stream()
+		elif self._renderer == EntryFormatter.GTKHTML:
+			self._document_lock.acquire()
+			imgs = IMG_REGEX.findall(html)
+			uncached=0
+			for url in imgs:
+				if not self._image_cache.is_cached(url):
+					uncached+=1
+					
+			if uncached > 0:
+				self._document.clear()
+				self._document.open_stream("text/html")
+				d = { 	"background_color": self._background_color,
+						"loading": _("Loading images...")}
+				self._document.write_stream("""<html><style type="text/css">
+		        body { background-color: %(background_color)s; }</style><body><i>%(loading)s</i></body></html>""" % d) 
+				self._document.close_stream()
+				self._document_lock.release()
+				
+				self._dl_count = 0
+				self._dl_total = uncached
+				
+				for url in imgs:
+					if not self._image_cache.is_cached(url):
+						self._image_pool.queueTask(self._gtkhtml_do_download_image, (url, self._current_feed_id, self._first_entry), self._gtkhtml_image_dl_cb)
+				self._image_pool.queueTask(self._gtkhtml_download_done, (self._current_feed_id, self._first_entry, html))
 			else:
-				self._moz.open_stream("file:///","text/html")
-			while len(html)>60000:
-				part = html[0:60000]
-				html = html[60000:]
-				self._moz.append_data(part, long(len(part)))
-			self._moz.append_data(html, long(len(html)))
-			self._moz.close_stream()
+				self._scrolled_window.get_hadjustment().set_value(0)
+				self._scrolled_window.get_vadjustment().set_value(0)
+				self._document.clear()
+				self._document.open_stream("text/html")
+				self._document.write_stream(html)
+				self._document.close_stream()
+				self._document_lock.release()
+				
+	def _gtkhtml_reset_image_dl(self):
+		assert self._renderer == EntryFormatter.GTKHTML
+		self._image_pool.joinAll(False, False)
+		self._dl_count = 0
+		self._dl_total = 0
+				
+	def _gtkhtml_do_download_image(self, args):
+		url, feed_id, first_entry = args
+		self._image_cache.get_image(url)
+		return (feed_id, first_entry)
+		
+	def _gtkhtml_image_dl_cb(self, args):
+		feed_id, first_entry = args
+		if feed_id == self._current_feed_id and first_entry == self._first_entry:
+			self._dl_count += 1
+			
+	def _gtkhtml_download_done(self, args):
+		feed_id, first_entry, html = args
+		
+		count = 0
+		last_count = self._dl_count
+		while feed_id == self._current_feed_id and first_entry == self._first_entry and count < (10 * 2):
+			if last_count != self._dl_count:
+				#if downloads are still coming in, reset counter
+				last_count = self._dl_count
+				count = 0
+			if self._dl_count >= self._dl_total:
+				gobject.idle_add(self._gtkhtml_images_loaded, feed_id, first_entry, html)
+				return
+			count += 1
+			time.sleep(0.5)
+		gobject.idle_add(self._gtkhtml_images_loaded, feed_id, first_entry, html)
+
+		
+	def _gtkhtml_images_loaded(self, feed_id, first_entry, html):
+		#if we're changing, nevermind.
+		#also make sure entry is the same and that we shouldn't be blanks
+		if feed_id == self._current_feed_id and first_entry == self._first_entry:
+			va = self._scrolled_window.get_vadjustment()
+			ha = self._scrolled_window.get_hadjustment()
+			self._document_lock.acquire()
+			self._document.clear()
+			self._document.open_stream("text/html")
+			self._document.write_stream(html)
+			self._document.close_stream()
+			self._document_lock.release()
+		return False
+		
+	def _gtkhtml_request_url(self, document, url, stream):
+		try:
+			image = self._image_cache.get_image(url)
+			stream.write(image)
+			stream.close()
+		except Exception, ex:
+			stream.close()
 			
 	def _do_delayed_set_viewed(self, feed_id, first_entry, last_entry, show_change=False):
 		if (feed_id, first_entry, last_entry) != \
@@ -849,6 +1073,13 @@ class PlanetView(gobject.GObject):
 			except ptvDB.NoEntry:
 				return
 				
+			item = gtk.MenuItem(_("_Open in Browser..."))
+			item.connect('activate', lambda e: self._app.activate_link(entry['link']))
+			menu.append(item)
+			
+			#separator = gtk.SeparatorMenuItem()
+			#menu.append(separator)
+				
 			entry['flag'] = self._db.get_entry_flag(entry_id)
 			
 			if entry['flag'] & ptvDB.F_MEDIA:
@@ -872,6 +1103,15 @@ class PlanetView(gobject.GObject):
 					item = gtk.MenuItem(_("Delete"))
 					item.connect('activate', lambda e,i: self._app.delete_entry_media(i), entry_id)
 					menu.append(item)
+					
+				if entry['flag'] & ptvDB.F_UNVIEWED:
+					item = gtk.MenuItem(_("Mark As _Viewed"))
+					item.connect('activate', lambda e,i: self._app.mark_entry_as_viewed(i), entry_id)
+					menu.append(item)
+				else:
+					item = gtk.MenuItem(_("Mark As _Unviewed"))
+					item.connect('activate', lambda e,i: self._app.mark_entry_as_unviewed(i), entry_id)
+					menu.append(item)
 			
 			keep = self._db.get_entry_keep(entry['entry_id'])
 			if keep:
@@ -888,13 +1128,13 @@ class PlanetView(gobject.GObject):
 				menu.append(separator)
 				
 		if self._state != S_SEARCH:
-			if self._show_kept:
+			if self._hide_viewed:
 				item = gtk.MenuItem(_("_Show All"))
-				item.connect('activate', self._toggle_show_kept)
+				item.connect('activate', self._toggle_hide_viewed)
 				menu.append(item)
 			else:
-				item = gtk.MenuItem(_("_Show Kept Entries"))
-				item.connect('activate', self._toggle_show_kept)
+				item = gtk.MenuItem(_("_Hide Viewed Entries"))
+				item.connect('activate', self._toggle_hide_viewed)
 				menu.append(item)
 			
 		menu.show_all()
@@ -902,31 +1142,48 @@ class PlanetView(gobject.GObject):
 		
 	def _moz_link_clicked(self, mozembed, link):
 		link = link.strip()
+		self._link_clicked(link)
+		return True #don't load url please
+		
+	def _link_clicked(self, link):
 		if link == "planet:up":
-			self._first_entry -= ENTRIES_PER_PAGE
-			self._render_entries(mark_read=True)
+			self._do_planet_up()
 		elif link == "planet:down":
-			self._first_entry += ENTRIES_PER_PAGE
-			self._render_entries(mark_read=True)
+			self._do_planet_down()
+		elif link == "pane:back":
+			self._do_pane_back()
 		elif link.startswith("rightclick"):
 			self._do_context_menu(int(link.split(':')[1]))
 		else:
 			self.emit('link-activated', link)
-			
-		return True #don't load url please
+
+	def _do_pane_back(self, a=None):
+		self._main_window.pane_to_feeds()
+	
+	def _do_planet_up(self, a=None):
+		self._first_entry -= ENTRIES_PER_PAGE
+		if self._renderer == EntryFormatter.GTKHTML:	
+			self._gtkhtml_reset_image_dl()
+		self._render_entries(mark_read=True)
 		
-	def set_show_kept(self, state):
-		if state == self._show_kept:
+	def _do_planet_down(self, a=None):
+		self._first_entry += ENTRIES_PER_PAGE
+		if self._renderer == EntryFormatter.GTKHTML:	
+			self._gtkhtml_reset_image_dl()
+		self._render_entries(mark_read=True)
+		
+	def set_hide_viewed(self, state):
+		if state == self._hide_viewed:
 			return
 				
-		self._toggle_show_kept()
+		self._toggle_hide_viewed()
 		
-	def _toggle_show_kept(self, e=None):
-		if self._show_kept:
-			self._show_kept = False
+	def _toggle_hide_viewed(self, e=None):
+		if self._hide_viewed:
+			self._hide_viewed = False
 		else:
-			self._show_kept = True
-		self._main_window.set_show_kept_menuitem(self._show_kept)
+			self._hide_viewed = True
+		self._main_window.set_hide_entries_menuitem(self._hide_viewed)
 		self._first_entry = 0
 		self._render_entries()
 		
@@ -941,6 +1198,15 @@ class PlanetView(gobject.GObject):
 	def _moz_link_message(self, data):
 		if not utils.RUNNING_HILDON:
 			self._main_window.display_status_message(self._moz.get_link_message())
+			
+	def _gtkhtml_link_clicked(self, document, link):
+		link = link.strip()
+		self._link_clicked(link)
+	
+	def _gtkhtml_on_url(self, view, url):
+		if url == None:
+			url = ""
+		self._main_window.display_status_message(url)
 	
 	def _gconf_reset_moz_font(self, client, *args, **kwargs):
 		self._reset_moz_font()
@@ -963,7 +1229,4 @@ class PlanetView(gobject.GObject):
 		self._moz_font = " ".join(map(str, [x for x in moz_font.split() if not isNumber(x)]))
 		self._moz_font = "'"+self._moz_font+"','"+" ".join(map(str, [x for x in moz_font.split() if isValid(x)])) + "',Arial"
 		self._moz_size = int([x for x in moz_font.split() if isNumber(x)][-1])+4
-		
-
-					
 
