@@ -6,24 +6,19 @@ import os,sys,time, pwd, os.path
 import subprocess
 import string
 import urllib
-import HTMLParser
 import string
 import locale
 import gettext
 import shutil
 import logging
 
+VERSION="4.0.0"
+
 try:
 	from sqlite3 import OperationalError as OperationalError
 except:
 	from pysqlite2.dbapi2 import OperationalError as OperationalError
 
-try:
-	import gtk
-	GTK_OK = True
-except:
-	GTK_OK = False
-	
 try:
 	import hildon
 	logging.debug("Detected hildon environment")
@@ -31,9 +26,20 @@ try:
 except:
 	RUNNING_HILDON = False
 
+try:
+	if RUNNING_HILDON:
+		DO_GTK_CHECKS = False
+	else:
+		import gtk
+		DO_GTK_CHECKS = True
+except:
+	DO_GTK_CHECKS = False
+	
+
+
 from subprocess import Popen, PIPE, STDOUT
 
-locale.setlocale(locale.LC_ALL, '')
+#locale.setlocale(locale.LC_ALL, '')
 gettext.install('penguintv', '/usr/share/locale')
 gettext.bindtextdomain('penguintv', '/usr/share/locale')
 gettext.textdomain('penguintv')
@@ -62,11 +68,17 @@ if RUNNING_SUGAR:
 	except:
 		HAS_GSTREAMER = False
 else:
-	try:
-		import gtkmozembed
-		HAS_MOZILLA = True
-	except:
-		HAS_MOZILLA = False
+	if DO_GTK_CHECKS:
+		try:
+			import gtkmozembed
+			HAS_MOZILLA = True
+		except:
+			try:
+				from ptvmozembed import gtkmozembed
+				logging.info("Using PenguinTV-built gtkmozembed")
+				HAS_MOZILLA = True
+			except:
+				HAS_MOZILLA = False
 
 	HAS_SEARCH = False
 
@@ -93,15 +105,16 @@ else:
 		except:
 			HAS_GCONF = False
 		
-	try:
-		import gnomevfs
-		HAS_GNOMEVFS = True
-	except:
+	if DO_GTK_CHECKS:
 		try:
-			from gnome import gnomevfs
+			import gnomevfs
 			HAS_GNOMEVFS = True
 		except:
-			HAS_GNOMEVFS = False
+			try:
+				from gnome import gnomevfs
+				HAS_GNOMEVFS = True
+			except:
+				HAS_GNOMEVFS = False
 		
 	try:
 		from xml.sax.handler import ContentHandler
@@ -113,7 +126,7 @@ else:
 		except:
 			HAS_PYXML = False
 		
-	if GTK_OK:
+	if DO_GTK_CHECKS and not RUNNING_HILDON:
 		if gtk.pygtk_version >= (2, 10, 0):
 			HAS_STATUS_ICON = True
 		else:
@@ -134,9 +147,11 @@ else:
 		HAS_GSTREAMER = False
 
 if RUNNING_HILDON:
+	#having a status icon causes tons of problems (causes hildonn UI to crash)
 	HAS_STATUS_ICON = False
+	HAS_MOZILLA = False
+	HAS_GNOMEVFS = False
 	
-VERSION="3.65"
 #DEBUG
 #_USE_KDE_OVERRIDE=False
 # Lucene sucks, forget it
@@ -146,13 +161,12 @@ HAS_LUCENE = False
 if not HAS_XAPIAN:
 	HAS_SEARCH = False
 	
-if GTK_OK:
-	if HAS_XAPIAN:
-		logging.info("Using Xapian search engine")
-	elif HAS_LUCENE:
-		logging.info("Using Lucene search engine")
-	else:
-		logging.info("xapian or lucene not found, search disabled")
+if HAS_XAPIAN:
+	logging.info("Using Xapian search engine")
+elif HAS_LUCENE:
+	logging.info("Using Lucene search engine")
+else:
+	logging.info("xapian or lucene not found, search disabled")
 	
 # Pynotify is still broken, forget it
 HAS_PYNOTIFY = False
@@ -161,6 +175,9 @@ HAS_PYNOTIFY = False
 #HAS_GNOMEVFS = False
 #HAS_MOZILLA=False
 
+#too buggy, disable for release
+ENABLE_ARTICLESYNC=False
+
 def get_home():
 	if RUNNING_SUGAR:
 		import sugar.env
@@ -186,35 +203,47 @@ def GetPrefix():
 	return h
 	
 _glade_prefix = None
+_share_prefix = None
+
+def get_share_prefix():	
+	global _share_prefix
+
+	if _share_prefix is not None:
+		return _share_prefix
+	
+	get_glade_prefix()
+	return _share_prefix
 	
 def get_glade_prefix():
 	global _glade_prefix
+	global _share_prefix
 	if _glade_prefix is not None:
 		return _glade_prefix
 
 	logging.debug("finding glade prefix")
 
 	import utils
-	for p in (os.path.join(GetPrefix(),"share","penguintv"),
-		  os.path.join(GetPrefix(),"share"),
+	for p in (os.path.join(GetPrefix(),"share","penguintv","glade"),
+		  os.path.join(GetPrefix(),"share","glade"),
 		  os.path.join(os.path.split(os.path.abspath(sys.argv[0]))[0],"share"),
 		  os.path.join(GetPrefix(),"share","sugar","activities","ptv","share"),
-		  os.path.join(os.path.split(os.path.split(utils.__file__)[0])[0],'share'),
-		  "/usr/share/penguintv"):
+		  os.path.join(os.path.split(os.path.split(utils.__file__)[0])[0],'share','glade'),
+		  "/usr/share/penguintv/glade"):
 		try:
-			os.stat(os.path.join(p,"penguintv.glade"))
+			os.stat(os.path.join(p,"dialogs.glade"))
 			logging.debug("glade prefix found: %s" % (p,))
 			_glade_prefix = p
+			_share_prefix = '/'.join(p.split('/')[0:-1])
 			return _glade_prefix
 		except:
 			continue
 	return None
 	
 def get_image_path(filename):
-	for p in (os.path.join(GetPrefix(), 'share', 'pixmaps'),
+	for p in (get_share_prefix(),
+			  os.path.join(GetPrefix(), 'share', 'pixmaps'),
 			  os.path.join(GetPrefix(), 'share'), #in case the install is still in the source dirs
-			  get_glade_prefix(),
-			  os.path.join(get_glade_prefix(), 'pixmaps')):
+			  os.path.join(get_share_prefix(), 'pixmaps')):
 		try:
 			icon_file = os.path.join(p, filename)
 			os.stat(icon_file)
@@ -375,6 +404,12 @@ def my_quote(s):
 	s=string.replace(s,"&","&amp;")
 	return s
 	
+def get_hyphen():
+	if RUNNING_HILDON:
+		return "--"
+	else:
+		return "&#8211;"
+	
 def uniquer(seq, idfun=None):
 	if not seq:
 		return []
@@ -446,60 +481,7 @@ def is_file_media(filename):
 #	def get_fed_data(self):
 #		return ''.join(self.fed)
 
-#http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/52281
-class StrippingParser(HTMLParser.HTMLParser):
-
-    # These are the HTML tags that we will leave intact
-    #valid_tags = ('b', 'a', 'i', 'br', 'p')
-    valid_tags = ('b', 'i', 'p')
-
-    from htmlentitydefs import entitydefs # replace entitydefs from sgmllib
-    
-    def __init__(self):
-        HTMLParser.HTMLParser.__init__(self)
-        self.result = ""
-        self.endTagList = [] 
-        
-    def handle_data(self, data):
-        if data:
-            self.result = self.result + data
-
-    def handle_charref(self, name):
-        self.result = "%s&#%s;" % (self.result, name)
-        
-    def handle_entityref(self, name):
-        if self.entitydefs.has_key(name): 
-            x = ';'
-        else:
-            # this breaks unstandard entities that end with ';'
-            x = ''
-        self.result = "%s&%s%s" % (self.result, name, x)
-    
-    #def handle_starttag(self, tag, attrs):
-    #    """ Delete all tags except for legal ones """
-    #    if tag in self.valid_tags:
-    #        #self.result = self.result + '<' + tag
-    #        for k, v in attrs:
-    #            if string.lower(k[0:2]) != 'on' and string.lower(v[0:10]) != 'javascript':
-    #                self.result = '%s %s="%s"' % (self.result, k, v)
-    #        #endTag = '</%s>' % tag
-    #        #self.endTagList.insert(0,endTag)    
-    #      	#self.result = self.result + '>'
-                
-    #def handle_endtag(self, tag):
-    #    if tag in self.valid_tags:
-    #    	
-    #        #self.result = "%s</%s>" % (self.result, tag)
-    #        #remTag = '</%s>' % tag
-    #        #try:
-    #        #	self.endTagList.remove(remTag)
-    #        #except:
-    #        #	pass
-
-    #def cleanup(self):
-    #    """ Append missing closing tags """
-    #    for j in range(len(self.endTagList)):
-    #            self.result = self.result + self.endTagList[j]    
+ 
         
 #usage:
 #def strip(s):
@@ -585,7 +567,8 @@ def init_gtkmozembed():
 	p = subprocess.Popen(cmd, shell=True, close_fds=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
 	retval = p.wait()
 	stderr = p.stderr.read()
-	if len(stderr) > 1 or retval == 0:
+	stdout = p.stdout.read()
+	if len(stderr) > 1 or len(stdout) > 0:
 		print """***ERROR initializing mozilla.  PenguinTV may crash shortly.  
 You may need to export LD_LIBRARY_PATH=$MOZILLA_FIVE_HOME
 """
